"""pinpoint base URL and path."""

from .responses import PinpointResponse

url_bases = [
    r"https?://pinpoint\.(.+)\.amazonaws\.com",
]


url_paths = {
    "{0}/v1/apps$": PinpointResponse.dispatch,
    "{0}/v1/apps/(?P<app_id>[^/]+)$": PinpointResponse.dispatch,
    "{0}/v1/apps/(?P<app_id>[^/]+)/eventstream": PinpointResponse.dispatch,
    "{0}/v1/apps/(?P<app_id>[^/]+)/settings$": PinpointResponse.dispatch,
    "{0}/v1/tags/(?P<app_arn>[^/]+)$": PinpointResponse.dispatch,
    "{0}/v1/tags/(?P<app_arn_pt_1>[^/]+)/(?P<app_arn_pt_2>[^/]+)$": PinpointResponse.dispatch,
}



from dataclasses import dataclass, field
from typing import Any, Dict, List, Literal, Optional

from moto.appmesh.dataclasses.shared import Metadata, Status
from moto.appmesh.dataclasses.virtual_node import VirtualNode
from moto.appmesh.dataclasses.virtual_router import VirtualRouter


@dataclass
class MeshSpec:
    egress_filter: Dict[Literal["type"], Optional[str]]
    service_discovery: Dict[Literal["ip_preference"], Optional[str]]


@dataclass
class Mesh:
    mesh_name: str
    metadata: Metadata
    spec: MeshSpec
    status: Status
    virtual_nodes: Dict[str, VirtualNode] = field(default_factory=dict)
    virtual_routers: Dict[str, VirtualRouter] = field(default_factory=dict)
    tags: List[Dict[str, str]] = field(default_factory=list)

    def to_dict(self) -> Dict[str, Any]:  # type ignore[misc]
        return {
            "meshName": self.mesh_name,
            "metadata": {
                "arn": self.metadata.arn,
                "createdAt": self.metadata.created_at.strftime("%d/%m/%Y, %H:%M:%S"),
                "lastUpdatedAt": self.metadata.last_updated_at.strftime(
                    "%d/%m/%Y, %H:%M:%S"
                ),
                "meshOwner": self.metadata.mesh_owner,
                "resourceOwner": self.metadata.resource_owner,
                "uid": self.metadata.uid,
                "version": self.metadata.version,
            },
            "spec": {
                "egressFilter": self.spec.egress_filter,
                "serviceDiscovery": {
                    "ipPreference": self.spec.service_discovery.get("ip_preference")
                },
            },
            "status": self.status,
        }



# Generated from ASLIntrinsicParser.g4 by ANTLR 4.13.2
# encoding: utf-8
import sys
from typing import TextIO

from antlr4 import (
    DFA,
    ATNDeserializer,
    NoViableAltException,
    Parser,
    ParserATNSimulator,
    ParserRuleContext,
    ParseTreeListener,
    ParseTreeVisitor,
    PredictionContextCache,
    RecognitionException,
    Token,
    TokenStream,
)


def serializedATN():
    return [
        4,
        1,
        34,
        46,
        2,
        0,
        7,
        0,
        2,
        1,
        7,
        1,
        2,
        2,
        7,
        2,
        2,
        3,
        7,
        3,
        2,
        4,
        7,
        4,
        1,
        0,
        1,
        0,
        1,
        0,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        1,
        2,
        1,
        2,
        1,
        3,
        1,
        3,
        1,
        3,
        1,
        3,
        5,
        3,
        25,
        8,
        3,
        10,
        3,
        12,
        3,
        28,
        9,
        3,
        1,
        3,
        1,
        3,
        1,
        3,
        1,
        3,
        3,
        3,
        34,
        8,
        3,
        1,
        4,
        1,
        4,
        1,
        4,
        1,
        4,
        1,
        4,
        1,
        4,
        1,
        4,
        1,
        4,
        3,
        4,
        44,
        8,
        4,
        1,
        4,
        0,
        0,
        5,
        0,
        2,
        4,
        6,
        8,
        0,
        2,
        1,
        0,
        12,
        29,
        1,
        0,
        9,
        10,
        49,
        0,
        10,
        1,
        0,
        0,
        0,
        2,
        13,
        1,
        0,
        0,
        0,
        4,
        18,
        1,
        0,
        0,
        0,
        6,
        33,
        1,
        0,
        0,
        0,
        8,
        43,
        1,
        0,
        0,
        0,
        10,
        11,
        3,
        2,
        1,
        0,
        11,
        12,
        5,
        0,
        0,
        1,
        12,
        1,
        1,
        0,
        0,
        0,
        13,
        14,
        5,
        11,
        0,
        0,
        14,
        15,
        5,
        8,
        0,
        0,
        15,
        16,
        3,
        4,
        2,
        0,
        16,
        17,
        3,
        6,
        3,
        0,
        17,
        3,
        1,
        0,
        0,
        0,
        18,
        19,
        7,
        0,
        0,
        0,
        19,
        5,
        1,
        0,
        0,
        0,
        20,
        21,
        5,
        5,
        0,
        0,
        21,
        26,
        3,
        8,
        4,
        0,
        22,
        23,
        5,
        7,
        0,
        0,
        23,
        25,
        3,
        8,
        4,
        0,
        24,
        22,
        1,
        0,
        0,
        0,
        25,
        28,
        1,
        0,
        0,
        0,
        26,
        24,
        1,
        0,
        0,
        0,
        26,
        27,
        1,
        0,
        0,
        0,
        27,
        29,
        1,
        0,
        0,
        0,
        28,
        26,
        1,
        0,
        0,
        0,
        29,
        30,
        5,
        6,
        0,
        0,
        30,
        34,
        1,
        0,
        0,
        0,
        31,
        32,
        5,
        5,
        0,
        0,
        32,
        34,
        5,
        6,
        0,
        0,
        33,
        20,
        1,
        0,
        0,
        0,
        33,
        31,
        1,
        0,
        0,
        0,
        34,
        7,
        1,
        0,
        0,
        0,
        35,
        44,
        5,
        30,
        0,
        0,
        36,
        44,
        5,
        31,
        0,
        0,
        37,
        44,
        5,
        32,
        0,
        0,
        38,
        44,
        7,
        1,
        0,
        0,
        39,
        44,
        5,
        1,
        0,
        0,
        40,
        44,
        5,
        2,
        0,
        0,
        41,
        44,
        5,
        3,
        0,
        0,
        42,
        44,
        3,
        2,
        1,
        0,
        43,
        35,
        1,
        0,
        0,
        0,
        43,
        36,
        1,
        0,
        0,
        0,
        43,
        37,
        1,
        0,
        0,
        0,
        43,
        38,
        1,
        0,
        0,
        0,
        43,
        39,
        1,
        0,
        0,
        0,
        43,
        40,
        1,
        0,
        0,
        0,
        43,
        41,
        1,
        0,
        0,
        0,
        43,
        42,
        1,
        0,
        0,
        0,
        44,
        9,
        1,
        0,
        0,
        0,
        3,
        26,
        33,
        43,
    ]


class ASLIntrinsicParser(Parser):
    grammarFileName = "ASLIntrinsicParser.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [DFA(ds, i) for i, ds in enumerate(atn.decisionToState)]

    sharedContextCache = PredictionContextCache()

    literalNames = [
        "<INVALID>",
        "<INVALID>",
        "<INVALID>",
        "<INVALID>",
        "'$'",
        "'('",
        "')'",
        "','",
        "'.'",
        "'true'",
        "'false'",
        "'States'",
        "'Format'",
        "'StringToJson'",
        "'JsonToString'",
        "'Array'",
        "'ArrayPartition'",
        "'ArrayContains'",
        "'ArrayRange'",
        "'ArrayGetItem'",
        "'ArrayLength'",
        "'ArrayUnique'",
        "'Base64Encode'",
        "'Base64Decode'",
        "'Hash'",
        "'JsonMerge'",
        "'MathRandom'",
        "'MathAdd'",
        "'StringSplit'",
        "'UUID'",
    ]

    symbolicNames = [
        "<INVALID>",
        "CONTEXT_PATH_STRING",
        "JSON_PATH_STRING",
        "STRING_VARIABLE",
        "DOLLAR",
        "LPAREN",
        "RPAREN",
        "COMMA",
        "DOT",
        "TRUE",
        "FALSE",
        "States",
        "Format",
        "StringToJson",
        "JsonToString",
        "Array",
        "ArrayPartition",
        "ArrayContains",
        "ArrayRange",
        "ArrayGetItem",
        "ArrayLength",
        "ArrayUnique",
        "Base64Encode",
        "Base64Decode",
        "Hash",
        "JsonMerge",
        "MathRandom",
        "MathAdd",
        "StringSplit",
        "UUID",
        "STRING",
        "INT",
        "NUMBER",
        "IDENTIFIER",
        "WS",
    ]

    RULE_func_decl = 0
    RULE_states_func_decl = 1
    RULE_state_fun_name = 2
    RULE_func_arg_list = 3
    RULE_func_arg = 4

    ruleNames = [
        "func_decl",
        "states_func_decl",
        "state_fun_name",
        "func_arg_list",
        "func_arg",
    ]

    EOF = Token.EOF
    CONTEXT_PATH_STRING = 1
    JSON_PATH_STRING = 2
    STRING_VARIABLE = 3
    DOLLAR = 4
    LPAREN = 5
    RPAREN = 6
    COMMA = 7
    DOT = 8
    TRUE = 9
    FALSE = 10
    States = 11
    Format = 12
    StringToJson = 13
    JsonToString = 14
    Array = 15
    ArrayPartition = 16
    ArrayContains = 17
    ArrayRange = 18
    ArrayGetItem = 19
    ArrayLength = 20
    ArrayUnique = 21
    Base64Encode = 22
    Base64Decode = 23
    Hash = 24
    JsonMerge = 25
    MathRandom = 26
    MathAdd = 27
    StringSplit = 28
    UUID = 29
    STRING = 30
    INT = 31
    NUMBER = 32
    IDENTIFIER = 33
    WS = 34

    def __init__(self, input: TokenStream, output: TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.13.2")
        self._interp = ParserATNSimulator(
            self, self.atn, self.decisionsToDFA, self.sharedContextCache
        )
        self._predicates = None

    class Func_declContext(ParserRuleContext):
        __slots__ = "parser"

        def __init__(
            self, parser, parent: ParserRuleContext = None, invokingState: int = -1
        ):
            super().__init__(parent, invokingState)
            self.parser = parser

        def states_func_decl(self):
            return self.getTypedRuleContext(
                ASLIntrinsicParser.States_func_declContext, 0
            )

        def EOF(self):
            return self.getToken(ASLIntrinsicParser.EOF, 0)

        def getRuleIndex(self):
            return ASLIntrinsicParser.RULE_func_decl

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterFunc_decl"):
                listener.enterFunc_decl(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitFunc_decl"):
                listener.exitFunc_decl(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitFunc_decl"):
                return visitor.visitFunc_decl(self)
            else:
                return visitor.visitChildren(self)

    def func_decl(self):
        localctx = ASLIntrinsicParser.Func_declContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_func_decl)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 10
            self.states_func_decl()
            self.state = 11
            self.match(ASLIntrinsicParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class States_func_declContext(ParserRuleContext):
        __slots__ = "parser"

        def __init__(
            self, parser, parent: ParserRuleContext = None, invokingState: int = -1
        ):
            super().__init__(parent, invokingState)
            self.parser = parser

        def States(self):
            return self.getToken(ASLIntrinsicParser.States, 0)

        def DOT(self):
            return self.getToken(ASLIntrinsicParser.DOT, 0)

        def state_fun_name(self):
            return self.getTypedRuleContext(ASLIntrinsicParser.State_fun_nameContext, 0)

        def func_arg_list(self):
            return self.getTypedRuleContext(ASLIntrinsicParser.Func_arg_listContext, 0)

        def getRuleIndex(self):
            return ASLIntrinsicParser.RULE_states_func_decl

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterStates_func_decl"):
                listener.enterStates_func_decl(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitStates_func_decl"):
                listener.exitStates_func_decl(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitStates_func_decl"):
                return visitor.visitStates_func_decl(self)
            else:
                return visitor.visitChildren(self)

    def states_func_decl(self):
        localctx = ASLIntrinsicParser.States_func_declContext(
            self, self._ctx, self.state
        )
        self.enterRule(localctx, 2, self.RULE_states_func_decl)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 13
            self.match(ASLIntrinsicParser.States)
            self.state = 14
            self.match(ASLIntrinsicParser.DOT)
            self.state = 15
            self.state_fun_name()
            self.state = 16
            self.func_arg_list()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class State_fun_nameContext(ParserRuleContext):
        __slots__ = "parser"

        def __init__(
            self, parser, parent: ParserRuleContext = None, invokingState: int = -1
        ):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Format(self):
            return self.getToken(ASLIntrinsicParser.Format, 0)

        def StringToJson(self):
            return self.getToken(ASLIntrinsicParser.StringToJson, 0)

        def JsonToString(self):
            return self.getToken(ASLIntrinsicParser.JsonToString, 0)

        def Array(self):
            return self.getToken(ASLIntrinsicParser.Array, 0)

        def ArrayPartition(self):
            return self.getToken(ASLIntrinsicParser.ArrayPartition, 0)

        def ArrayContains(self):
            return self.getToken(ASLIntrinsicParser.ArrayContains, 0)

        def ArrayRange(self):
            return self.getToken(ASLIntrinsicParser.ArrayRange, 0)

        def ArrayGetItem(self):
            return self.getToken(ASLIntrinsicParser.ArrayGetItem, 0)

        def ArrayLength(self):
            return self.getToken(ASLIntrinsicParser.ArrayLength, 0)

        def ArrayUnique(self):
            return self.getToken(ASLIntrinsicParser.ArrayUnique, 0)

        def Base64Encode(self):
            return self.getToken(ASLIntrinsicParser.Base64Encode, 0)

        def Base64Decode(self):
            return self.getToken(ASLIntrinsicParser.Base64Decode, 0)

        def Hash(self):
            return self.getToken(ASLIntrinsicParser.Hash, 0)

        def JsonMerge(self):
            return self.getToken(ASLIntrinsicParser.JsonMerge, 0)

        def MathRandom(self):
            return self.getToken(ASLIntrinsicParser.MathRandom, 0)

        def MathAdd(self):
            return self.getToken(ASLIntrinsicParser.MathAdd, 0)

        def StringSplit(self):
            return self.getToken(ASLIntrinsicParser.StringSplit, 0)

        def UUID(self):
            return self.getToken(ASLIntrinsicParser.UUID, 0)

        def getRuleIndex(self):
            return ASLIntrinsicParser.RULE_state_fun_name

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterState_fun_name"):
                listener.enterState_fun_name(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitState_fun_name"):
                listener.exitState_fun_name(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitState_fun_name"):
                return visitor.visitState_fun_name(self)
            else:
                return visitor.visitChildren(self)

    def state_fun_name(self):
        localctx = ASLIntrinsicParser.State_fun_nameContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_state_fun_name)
        self._la = 0  # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 18
            _la = self._input.LA(1)
            if not (((_la) & ~0x3F) == 0 and ((1 << _la) & 1073737728) != 0):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Func_arg_listContext(ParserRuleContext):
        __slots__ = "parser"

        def __init__(
            self, parser, parent: ParserRuleContext = None, invokingState: int = -1
        ):
            super().__init__(parent, invokingState)
            self.parser = parser

        def LPAREN(self):
            return self.getToken(ASLIntrinsicParser.LPAREN, 0)

        def func_arg(self, i: int = None):
            if i is None:
                return self.getTypedRuleContexts(ASLIntrinsicParser.Func_argContext)
            else:
                return self.getTypedRuleContext(ASLIntrinsicParser.Func_argContext, i)

        def RPAREN(self):
            return self.getToken(ASLIntrinsicParser.RPAREN, 0)

        def COMMA(self, i: int = None):
            if i is None:
                return self.getTokens(ASLIntrinsicParser.COMMA)
            else:
                return self.getToken(ASLIntrinsicParser.COMMA, i)

        def getRuleIndex(self):
            return ASLIntrinsicParser.RULE_func_arg_list

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterFunc_arg_list"):
                listener.enterFunc_arg_list(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitFunc_arg_list"):
                listener.exitFunc_arg_list(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitFunc_arg_list"):
                return visitor.visitFunc_arg_list(self)
            else:
                return visitor.visitChildren(self)

    def func_arg_list(self):
        localctx = ASLIntrinsicParser.Func_arg_listContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_func_arg_list)
        self._la = 0  # Token type
        try:
            self.state = 33
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input, 1, self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 20
                self.match(ASLIntrinsicParser.LPAREN)
                self.state = 21
                self.func_arg()
                self.state = 26
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la == 7:
                    self.state = 22
                    self.match(ASLIntrinsicParser.COMMA)
                    self.state = 23
                    self.func_arg()
                    self.state = 28
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 29
                self.match(ASLIntrinsicParser.RPAREN)
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 31
                self.match(ASLIntrinsicParser.LPAREN)
                self.state = 32
                self.match(ASLIntrinsicParser.RPAREN)
                pass

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx

    class Func_argContext(ParserRuleContext):
        __slots__ = "parser"

        def __init__(
            self, parser, parent: ParserRuleContext = None, invokingState: int = -1
        ):
            super().__init__(parent, invokingState)
            self.parser = parser

        def getRuleIndex(self):
            return ASLIntrinsicParser.RULE_func_arg

        def copyFrom(self, ctx: ParserRuleContext):
            super().copyFrom(ctx)

    class Func_arg_context_pathContext(Func_argContext):
        def __init__(
            self, parser, ctx: ParserRuleContext
        ):  # actually a ASLIntrinsicParser.Func_argContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def CONTEXT_PATH_STRING(self):
            return self.getToken(ASLIntrinsicParser.CONTEXT_PATH_STRING, 0)

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterFunc_arg_context_path"):
                listener.enterFunc_arg_context_path(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitFunc_arg_context_path"):
                listener.exitFunc_arg_context_path(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitFunc_arg_context_path"):
                return visitor.visitFunc_arg_context_path(self)
            else:
                return visitor.visitChildren(self)

    class Func_arg_floatContext(Func_argContext):
        def __init__(
            self, parser, ctx: ParserRuleContext
        ):  # actually a ASLIntrinsicParser.Func_argContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def NUMBER(self):
            return self.getToken(ASLIntrinsicParser.NUMBER, 0)

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterFunc_arg_float"):
                listener.enterFunc_arg_float(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitFunc_arg_float"):
                listener.exitFunc_arg_float(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitFunc_arg_float"):
                return visitor.visitFunc_arg_float(self)
            else:
                return visitor.visitChildren(self)

    class Func_arg_varContext(Func_argContext):
        def __init__(
            self, parser, ctx: ParserRuleContext
        ):  # actually a ASLIntrinsicParser.Func_argContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def STRING_VARIABLE(self):
            return self.getToken(ASLIntrinsicParser.STRING_VARIABLE, 0)

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterFunc_arg_var"):
                listener.enterFunc_arg_var(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitFunc_arg_var"):
                listener.exitFunc_arg_var(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitFunc_arg_var"):
                return visitor.visitFunc_arg_var(self)
            else:
                return visitor.visitChildren(self)

    class Func_arg_func_declContext(Func_argContext):
        def __init__(
            self, parser, ctx: ParserRuleContext
        ):  # actually a ASLIntrinsicParser.Func_argContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def states_func_decl(self):
            return self.getTypedRuleContext(
                ASLIntrinsicParser.States_func_declContext, 0
            )

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterFunc_arg_func_decl"):
                listener.enterFunc_arg_func_decl(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitFunc_arg_func_decl"):
                listener.exitFunc_arg_func_decl(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitFunc_arg_func_decl"):
                return visitor.visitFunc_arg_func_decl(self)
            else:
                return visitor.visitChildren(self)

    class Func_arg_intContext(Func_argContext):
        def __init__(
            self, parser, ctx: ParserRuleContext
        ):  # actually a ASLIntrinsicParser.Func_argContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def INT(self):
            return self.getToken(ASLIntrinsicParser.INT, 0)

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterFunc_arg_int"):
                listener.enterFunc_arg_int(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitFunc_arg_int"):
                listener.exitFunc_arg_int(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitFunc_arg_int"):
                return visitor.visitFunc_arg_int(self)
            else:
                return visitor.visitChildren(self)

    class Func_arg_boolContext(Func_argContext):
        def __init__(
            self, parser, ctx: ParserRuleContext
        ):  # actually a ASLIntrinsicParser.Func_argContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def TRUE(self):
            return self.getToken(ASLIntrinsicParser.TRUE, 0)

        def FALSE(self):
            return self.getToken(ASLIntrinsicParser.FALSE, 0)

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterFunc_arg_bool"):
                listener.enterFunc_arg_bool(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitFunc_arg_bool"):
                listener.exitFunc_arg_bool(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitFunc_arg_bool"):
                return visitor.visitFunc_arg_bool(self)
            else:
                return visitor.visitChildren(self)

    class Func_arg_stringContext(Func_argContext):
        def __init__(
            self, parser, ctx: ParserRuleContext
        ):  # actually a ASLIntrinsicParser.Func_argContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def STRING(self):
            return self.getToken(ASLIntrinsicParser.STRING, 0)

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterFunc_arg_string"):
                listener.enterFunc_arg_string(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitFunc_arg_string"):
                listener.exitFunc_arg_string(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitFunc_arg_string"):
                return visitor.visitFunc_arg_string(self)
            else:
                return visitor.visitChildren(self)

    class Func_arg_json_pathContext(Func_argContext):
        def __init__(
            self, parser, ctx: ParserRuleContext
        ):  # actually a ASLIntrinsicParser.Func_argContext
            super().__init__(parser)
            self.copyFrom(ctx)

        def JSON_PATH_STRING(self):
            return self.getToken(ASLIntrinsicParser.JSON_PATH_STRING, 0)

        def enterRule(self, listener: ParseTreeListener):
            if hasattr(listener, "enterFunc_arg_json_path"):
                listener.enterFunc_arg_json_path(self)

        def exitRule(self, listener: ParseTreeListener):
            if hasattr(listener, "exitFunc_arg_json_path"):
                listener.exitFunc_arg_json_path(self)

        def accept(self, visitor: ParseTreeVisitor):
            if hasattr(visitor, "visitFunc_arg_json_path"):
                return visitor.visitFunc_arg_json_path(self)
            else:
                return visitor.visitChildren(self)

    def func_arg(self):
        localctx = ASLIntrinsicParser.Func_argContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_func_arg)
        self._la = 0  # Token type
        try:
            self.state = 43
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [30]:
                localctx = ASLIntrinsicParser.Func_arg_stringContext(self, localctx)
                self.enterOuterAlt(localctx, 1)
                self.state = 35
                self.match(ASLIntrinsicParser.STRING)
                pass
            elif token in [31]:
                localctx = ASLIntrinsicParser.Func_arg_intContext(self, localctx)
                self.enterOuterAlt(localctx, 2)
                self.state = 36
                self.match(ASLIntrinsicParser.INT)
                pass
            elif token in [32]:
                localctx = ASLIntrinsicParser.Func_arg_floatContext(self, localctx)
                self.enterOuterAlt(localctx, 3)
                self.state = 37
                self.match(ASLIntrinsicParser.NUMBER)
                pass
            elif token in [9, 10]:
                localctx = ASLIntrinsicParser.Func_arg_boolContext(self, localctx)
                self.enterOuterAlt(localctx, 4)
                self.state = 38
                _la = self._input.LA(1)
                if not (_la == 9 or _la == 10):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                pass
            elif token in [1]:
                localctx = ASLIntrinsicParser.Func_arg_context_pathContext(
                    self, localctx
                )
                self.enterOuterAlt(localctx, 5)
                self.state = 39
                self.match(ASLIntrinsicParser.CONTEXT_PATH_STRING)
                pass
            elif token in [2]:
                localctx = ASLIntrinsicParser.Func_arg_json_pathContext(self, localctx)
                self.enterOuterAlt(localctx, 6)
                self.state = 40
                self.match(ASLIntrinsicParser.JSON_PATH_STRING)
                pass
            elif token in [3]:
                localctx = ASLIntrinsicParser.Func_arg_varContext(self, localctx)
                self.enterOuterAlt(localctx, 7)
                self.state = 41
                self.match(ASLIntrinsicParser.STRING_VARIABLE)
                pass
            elif token in [11]:
                localctx = ASLIntrinsicParser.Func_arg_func_declContext(self, localctx)
                self.enterOuterAlt(localctx, 8)
                self.state = 42
                self.states_func_decl()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx



"""
Ensure that the responses module plays nice with our mocks
"""

from http.server import BaseHTTPRequestHandler, HTTPServer
from threading import Event, Thread
from typing import Optional, Tuple
from unittest import SkipTest, TestCase

import boto3
import requests
import responses

from moto import mock_aws, settings
from moto.core.models import override_responses_real_send
from moto.core.versions import RESPONSES_VERSION
from moto.utilities.distutils_version import LooseVersion


class TestResponsesModule(TestCase):
    def setUp(self) -> None:
        if settings.TEST_SERVER_MODE:
            raise SkipTest("No point in testing responses-decorator in ServerMode")

    @mock_aws
    @responses.activate
    def test_moto_first(self) -> None:  # type: ignore[misc]
        """
        Verify we can activate a user-defined `responses` on top of our Moto mocks
        """
        self.moto_responses_compatibility()

    @responses.activate
    @mock_aws
    def test_moto_second(self) -> None:
        """
        Verify we can load Moto after activating a `responses`-mock
        """
        self.moto_responses_compatibility()

    def moto_responses_compatibility(self) -> None:
        responses.add(
            responses.GET, url="http://127.0.0.1/lkdsfjlkdsa", json={"a": "4"}
        )
        s3 = boto3.client("s3", region_name="us-east-1")
        s3.create_bucket(Bucket="mybucket")
        s3.put_object(Bucket="mybucket", Key="name", Body="value")
        s3.get_object(Bucket="mybucket", Key="name")["Body"].read()
        with requests.get("http://127.0.0.1/lkdsfjlkdsa") as r:
            assert r.json() == {"a": "4"}

    @responses.activate
    def test_moto_as_late_as_possible(self) -> None:
        """
        Verify we can load moto after registering a response
        """
        responses.add(
            responses.GET, url="http://127.0.0.1/lkdsfjlkdsa", json={"a": "4"}
        )
        with mock_aws():
            s3 = boto3.client("s3", region_name="us-east-1")
            s3.create_bucket(Bucket="mybucket")
            s3.put_object(Bucket="mybucket", Key="name", Body="value")
            # This mock exists within Moto
            with requests.get("http://127.0.0.1/lkdsfjlkdsa") as r:
                assert r.json() == {"a": "4"}

        # And outside of Moto
        with requests.get("http://127.0.0.1/lkdsfjlkdsa") as r:
            assert r.json() == {"a": "4"}


class WebRequestHandler(BaseHTTPRequestHandler):
    def do_GET(self) -> None:
        self.send_response(200)
        self.send_header("Content-Type", "application/json")
        self.end_headers()
        self.wfile.write(b"real response")


class SimpleServer:
    def __init__(self) -> None:
        self._port = 0

        self._thread: Optional[Thread] = None
        self._ip_address = "0.0.0.0"
        self._server: Optional[HTTPServer] = None
        self._server_ready_event = Event()

    def _server_entry(self) -> None:
        self._server = HTTPServer(("0.0.0.0", 0), WebRequestHandler)
        self._server_ready_event.set()
        self._server.serve_forever()

    def start(self) -> None:
        self._thread = Thread(target=self._server_entry, daemon=True)
        self._thread.start()
        self._server_ready_event.wait()

    def get_host_and_port(self) -> Tuple[str, int]:
        assert self._server
        host, port = self._server.server_address
        return (str(host), port)

    def stop(self) -> None:
        if self._server:
            self._server.shutdown()
        if self._thread:
            self._thread.join()


@mock_aws
class TestResponsesMockWithPassThru(TestCase):
    """
    https://github.com/getmoto/moto/issues/6417
    """

    def setUp(self) -> None:
        if RESPONSES_VERSION < LooseVersion("0.24.0"):
            raise SkipTest("Can only test this with responses >= 0.24.0")

        self.server = SimpleServer()
        self.server.start()
        host, port = self.server.get_host_and_port()
        self.server_url = f"http://{host}:{port}"

        self.r_mock = responses.RequestsMock(assert_all_requests_are_fired=True)
        override_responses_real_send(self.r_mock)
        self.r_mock.start()
        self.r_mock.add_passthru(self.server_url)

    def tearDown(self) -> None:
        self.r_mock.stop()  # type: ignore
        self.r_mock.reset()  # type: ignore
        override_responses_real_send(None)

        self.server.stop()

    def http_requests(self) -> str:
        # Mock this website
        requests.post("https://example.org")

        # Passthrough this website
        assert requests.get(self.server_url).content == b"real response"

        return "OK"

    def aws_and_http_requests(self) -> str:
        ddb = boto3.client("dynamodb", "us-east-1")
        assert ddb.list_tables()["TableNames"] == []
        self.http_requests()
        return "OK"

    def test_http_requests(self) -> None:
        self.r_mock.add(responses.POST, "https://example.org", status=200)  # type: ignore
        self.assertEqual("OK", self.http_requests())

    def test_aws_and_http_requests(self) -> None:
        self.r_mock.add(responses.POST, "https://example.org", status=200)  # type: ignore
        self.assertEqual("OK", self.aws_and_http_requests())



import json

import boto3
import pytest
from boto3 import Session
from botocore.client import ClientError

from moto import mock_aws, settings

# All tests for s3-control cannot be run under the server without a modification of the
# hosts file on your system. This is due to the fact that the URL to the host is in the form of:
# ACCOUNT_ID.s3-control.amazonaws.com <-- That Account ID part is the problem. If you want to
# make use of the moto server, update your hosts file for `THE_ACCOUNT_ID_FOR_MOTO.localhost`
# and this will work fine.

if not settings.TEST_SERVER_MODE:

    @mock_aws
    def test_config_list_account_pab():
        from moto.core import DEFAULT_ACCOUNT_ID as ACCOUNT_ID

        client = boto3.client("s3control", region_name="us-west-2")
        config_client = boto3.client("config", region_name="us-west-2")

        # Create the aggregator:
        account_aggregation_source = {
            "AccountIds": [ACCOUNT_ID],
            "AllAwsRegions": True,
        }
        config_client.put_configuration_aggregator(
            ConfigurationAggregatorName="testing",
            AccountAggregationSources=[account_aggregation_source],
        )

        # Without a PAB in place:
        result = config_client.list_discovered_resources(
            resourceType="AWS::S3::AccountPublicAccessBlock"
        )
        assert not result["resourceIdentifiers"]
        result = config_client.list_aggregate_discovered_resources(
            ResourceType="AWS::S3::AccountPublicAccessBlock",
            ConfigurationAggregatorName="testing",
        )
        assert not result["ResourceIdentifiers"]

        # Create a PAB:
        client.put_public_access_block(
            AccountId=ACCOUNT_ID,
            PublicAccessBlockConfiguration={
                "BlockPublicAcls": True,
                "IgnorePublicAcls": True,
                "BlockPublicPolicy": True,
                "RestrictPublicBuckets": True,
            },
        )

        # Test that successful queries work (non-aggregated):
        result = config_client.list_discovered_resources(
            resourceType="AWS::S3::AccountPublicAccessBlock"
        )
        assert result["resourceIdentifiers"] == [
            {
                "resourceType": "AWS::S3::AccountPublicAccessBlock",
                "resourceId": ACCOUNT_ID,
            }
        ]
        result = config_client.list_discovered_resources(
            resourceType="AWS::S3::AccountPublicAccessBlock",
            resourceIds=[ACCOUNT_ID, "nope"],
        )
        assert result["resourceIdentifiers"] == [
            {
                "resourceType": "AWS::S3::AccountPublicAccessBlock",
                "resourceId": ACCOUNT_ID,
            }
        ]
        result = config_client.list_discovered_resources(
            resourceType="AWS::S3::AccountPublicAccessBlock", resourceName=""
        )
        assert result["resourceIdentifiers"] == [
            {
                "resourceType": "AWS::S3::AccountPublicAccessBlock",
                "resourceId": ACCOUNT_ID,
            }
        ]

        # Test that successful queries work (aggregated):
        result = config_client.list_aggregate_discovered_resources(
            ResourceType="AWS::S3::AccountPublicAccessBlock",
            ConfigurationAggregatorName="testing",
        )
        regions = set(Session().get_available_regions("config"))
        for resource in result["ResourceIdentifiers"]:
            regions.remove(resource.pop("SourceRegion"))
            assert resource == {
                "ResourceType": "AWS::S3::AccountPublicAccessBlock",
                "SourceAccountId": ACCOUNT_ID,
                "ResourceId": ACCOUNT_ID,
            }

        # Just check that the len is the same -- this should be reasonable
        regions = set(Session().get_available_regions("config"))
        result = config_client.list_aggregate_discovered_resources(
            ResourceType="AWS::S3::AccountPublicAccessBlock",
            ConfigurationAggregatorName="testing",
            Filters={"ResourceName": ""},
        )
        assert len(regions) == len(result["ResourceIdentifiers"])
        result = config_client.list_aggregate_discovered_resources(
            ResourceType="AWS::S3::AccountPublicAccessBlock",
            ConfigurationAggregatorName="testing",
            Filters={"ResourceName": "", "ResourceId": ACCOUNT_ID},
        )
        assert len(regions) == len(result["ResourceIdentifiers"])
        result = config_client.list_aggregate_discovered_resources(
            ResourceType="AWS::S3::AccountPublicAccessBlock",
            ConfigurationAggregatorName="testing",
            Filters={
                "ResourceName": "",
                "ResourceId": ACCOUNT_ID,
                "Region": "us-west-2",
            },
        )
        assert (
            result["ResourceIdentifiers"][0]["SourceRegion"] == "us-west-2"
            and len(result["ResourceIdentifiers"]) == 1
        )

        # Test aggregator pagination:
        result = config_client.list_aggregate_discovered_resources(
            ResourceType="AWS::S3::AccountPublicAccessBlock",
            ConfigurationAggregatorName="testing",
            Limit=1,
        )
        regions = sorted(set(Session().get_available_regions("config")))
        assert result["ResourceIdentifiers"][0] == {
            "ResourceType": "AWS::S3::AccountPublicAccessBlock",
            "SourceAccountId": ACCOUNT_ID,
            "ResourceId": ACCOUNT_ID,
            "SourceRegion": regions[0],
        }
        assert result["NextToken"] == regions[1]

        # Get the next region:
        result = config_client.list_aggregate_discovered_resources(
            ResourceType="AWS::S3::AccountPublicAccessBlock",
            ConfigurationAggregatorName="testing",
            Limit=1,
            NextToken=regions[1],
        )
        assert result["ResourceIdentifiers"][0] == {
            "ResourceType": "AWS::S3::AccountPublicAccessBlock",
            "SourceAccountId": ACCOUNT_ID,
            "ResourceId": ACCOUNT_ID,
            "SourceRegion": regions[1],
        }

        # Non-aggregated with incorrect info:
        result = config_client.list_discovered_resources(
            resourceType="AWS::S3::AccountPublicAccessBlock", resourceName="nope"
        )
        assert not result["resourceIdentifiers"]
        result = config_client.list_discovered_resources(
            resourceType="AWS::S3::AccountPublicAccessBlock", resourceIds=["nope"]
        )
        assert not result["resourceIdentifiers"]

        # Aggregated with incorrect info:
        result = config_client.list_aggregate_discovered_resources(
            ResourceType="AWS::S3::AccountPublicAccessBlock",
            ConfigurationAggregatorName="testing",
            Filters={"ResourceName": "nope"},
        )
        assert not result["ResourceIdentifiers"]
        result = config_client.list_aggregate_discovered_resources(
            ResourceType="AWS::S3::AccountPublicAccessBlock",
            ConfigurationAggregatorName="testing",
            Filters={"ResourceId": "nope"},
        )
        assert not result["ResourceIdentifiers"]
        result = config_client.list_aggregate_discovered_resources(
            ResourceType="AWS::S3::AccountPublicAccessBlock",
            ConfigurationAggregatorName="testing",
            Filters={"Region": "Nope"},
        )
        assert not result["ResourceIdentifiers"]

    @mock_aws
    def test_config_get_account_pab():
        from moto.core import DEFAULT_ACCOUNT_ID as ACCOUNT_ID

        client = boto3.client("s3control", region_name="us-west-2")
        config_client = boto3.client("config", region_name="us-west-2")

        # Create the aggregator:
        account_aggregation_source = {
            "AccountIds": [ACCOUNT_ID],
            "AllAwsRegions": True,
        }
        config_client.put_configuration_aggregator(
            ConfigurationAggregatorName="testing",
            AccountAggregationSources=[account_aggregation_source],
        )

        # Without a PAB in place:
        with pytest.raises(ClientError) as ce_err:
            config_client.get_resource_config_history(
                resourceType="AWS::S3::AccountPublicAccessBlock", resourceId=ACCOUNT_ID
            )
        assert (
            ce_err.value.response["Error"]["Code"] == "ResourceNotDiscoveredException"
        )
        # aggregate
        result = config_client.batch_get_resource_config(
            resourceKeys=[
                {
                    "resourceType": "AWS::S3::AccountPublicAccessBlock",
                    "resourceId": "ACCOUNT_ID",
                }
            ]
        )
        assert not result["baseConfigurationItems"]
        result = config_client.batch_get_aggregate_resource_config(
            ConfigurationAggregatorName="testing",
            ResourceIdentifiers=[
                {
                    "SourceAccountId": ACCOUNT_ID,
                    "SourceRegion": "us-west-2",
                    "ResourceId": ACCOUNT_ID,
                    "ResourceType": "AWS::S3::AccountPublicAccessBlock",
                    "ResourceName": "",
                }
            ],
        )
        assert not result["BaseConfigurationItems"]

        # Create a PAB:
        client.put_public_access_block(
            AccountId=ACCOUNT_ID,
            PublicAccessBlockConfiguration={
                "BlockPublicAcls": True,
                "IgnorePublicAcls": True,
                "BlockPublicPolicy": True,
                "RestrictPublicBuckets": True,
            },
        )

        # Get the proper config:
        proper_config = {
            "blockPublicAcls": True,
            "ignorePublicAcls": True,
            "blockPublicPolicy": True,
            "restrictPublicBuckets": True,
        }
        result = config_client.get_resource_config_history(
            resourceType="AWS::S3::AccountPublicAccessBlock", resourceId=ACCOUNT_ID
        )
        assert (
            json.loads(result["configurationItems"][0]["configuration"])
            == proper_config
        )
        assert (
            result["configurationItems"][0]["accountId"]
            == result["configurationItems"][0]["resourceId"]
            == ACCOUNT_ID
        )
        result = config_client.batch_get_resource_config(
            resourceKeys=[
                {
                    "resourceType": "AWS::S3::AccountPublicAccessBlock",
                    "resourceId": ACCOUNT_ID,
                }
            ]
        )
        assert len(result["baseConfigurationItems"]) == 1
        assert (
            json.loads(result["baseConfigurationItems"][0]["configuration"])
            == proper_config
        )
        assert (
            result["baseConfigurationItems"][0]["accountId"]
            == result["baseConfigurationItems"][0]["resourceId"]
            == ACCOUNT_ID
        )

        for region in Session().get_available_regions("s3control"):
            result = config_client.batch_get_aggregate_resource_config(
                ConfigurationAggregatorName="testing",
                ResourceIdentifiers=[
                    {
                        "SourceAccountId": ACCOUNT_ID,
                        "SourceRegion": region,
                        "ResourceId": ACCOUNT_ID,
                        "ResourceType": "AWS::S3::AccountPublicAccessBlock",
                        "ResourceName": "",
                    }
                ],
            )
            assert len(result["BaseConfigurationItems"]) == 1
            assert (
                json.loads(result["BaseConfigurationItems"][0]["configuration"])
                == proper_config
            )



import re

from moto.utilities.utils import ARN_PARTITION_REGEX, get_partition


def test_get_partition():
    assert get_partition(None) == "aws"
    assert get_partition("unknown") == "aws"
    assert get_partition("cn-north-1") == "aws-cn"


def test_partition_regex():
    assert re.match(ARN_PARTITION_REGEX, "arn:aws:someservice")
    assert re.match(ARN_PARTITION_REGEX, "arn:aws-cn:someservice")
    assert not re.match(ARN_PARTITION_REGEX, "arn:partition:someservice")
    assert not re.match(ARN_PARTITION_REGEX, "someservice")



import copy
from typing import Any, Optional, Set, TypedDict

from moto.stepfunctions.parser.asl.jsonata.jsonata import (
    VariableDeclarations,
    VariableReference,
    encode_jsonata_variable_declarations,
)
from moto.stepfunctions.parser.asl.utils.json_path import extract_json
from moto.stepfunctions.parser.utils import long_uid

_STATES_PREFIX: str = "$states"
_STATES_INPUT_PREFIX: str = "$states.input"
_STATES_CONTEXT_PREFIX: str = "$states.context"
_STATES_RESULT_PREFIX: str = "$states.result"
_STATES_ERROR_OUTPUT_PREFIX: str = "$states.errorOutput"


class ExecutionData(TypedDict):
    Id: str
    Input: Optional[Any]
    Name: str
    RoleArn: str
    StartTime: str  # Format: ISO 8601.


class StateData(TypedDict):
    EnteredTime: str  # Format: ISO 8601.
    Name: str
    RetryCount: int


class StateMachineData(TypedDict):
    Id: str
    Name: str


class TaskData(TypedDict):
    Token: str


class ItemData(TypedDict):
    # Contains the index number for the array item that is being currently processed.
    Index: int
    # Contains the array item being processed.
    Value: Optional[Any]


class MapData(TypedDict):
    Item: ItemData


class ContextObjectData(TypedDict):
    Execution: ExecutionData
    State: Optional[StateData]
    StateMachine: StateMachineData
    Task: Optional[TaskData]  # Null if the Parameters field is outside a task state.
    Map: Optional[MapData]  # Only available when processing a Map state.


class ContextObject:
    context_object_data: ContextObjectData

    def __init__(self, context_object: ContextObjectData):
        self.context_object_data = context_object

    def update_task_token(self) -> str:
        new_token = long_uid()
        self.context_object_data["Task"] = TaskData(Token=new_token)
        return new_token


class StatesData(TypedDict):
    input: Any
    context: ContextObjectData
    result: Optional[Optional[Any]]
    errorOutput: Optional[Optional[Any]]


class States:
    _states_data: StatesData
    context_object: ContextObject

    def __init__(self, context: ContextObjectData):
        input_value = context["Execution"]["Input"]
        self._states_data = StatesData(input=input_value, context=context)
        self.context_object = ContextObject(context_object=context)

    @staticmethod
    def _extract(query: Optional[str], data: Any) -> Any:
        if query is None:
            result = data
        else:
            result = extract_json(query, data)
        return copy.deepcopy(result)

    def extract(self, query: str) -> Any:
        if not query.startswith(_STATES_PREFIX):
            raise RuntimeError(f"No such variable {query} in $states")
        jsonpath_states_query = "$." + query[1:]
        return self._extract(jsonpath_states_query, self._states_data)

    def get_input(self, query: Optional[str] = None) -> Any:
        return self._extract(query, self._states_data["input"])

    def reset(self, input_value: Any) -> None:
        clone_input_value = copy.deepcopy(input_value)
        self._states_data["input"] = clone_input_value
        self._states_data["result"] = None
        self._states_data["errorOutput"] = None

    def get_context(self, query: Optional[str] = None) -> Any:
        return self._extract(query, self._states_data["context"])

    def get_result(self, query: Optional[str] = None) -> Any:
        if "result" not in self._states_data:
            raise RuntimeError("Illegal access to $states.result")
        return self._extract(query, self._states_data["result"])

    def set_result(self, result: Any) -> Any:
        clone_result = copy.deepcopy(result)
        self._states_data["result"] = clone_result

    def get_error_output(self, query: Optional[str] = None) -> Any:
        if "errorOutput" not in self._states_data:
            raise RuntimeError("Illegal access to $states.errorOutput")
        return self._extract(query, self._states_data["errorOutput"])

    def set_error_output(self, error_output: Any) -> None:
        clone_error_output = copy.deepcopy(error_output)
        self._states_data["errorOutput"] = clone_error_output

    def to_variable_declarations(
        self, variable_references: Optional[Set[VariableReference]] = None
    ) -> VariableDeclarations:
        if not variable_references or _STATES_PREFIX in variable_references:
            return encode_jsonata_variable_declarations(
                bindings={_STATES_PREFIX: self._states_data}
            )
        candidate_sub_states = {
            "input": _STATES_INPUT_PREFIX,
            "context": _STATES_CONTEXT_PREFIX,
            "result": _STATES_RESULT_PREFIX,
            "errorOutput": _STATES_ERROR_OUTPUT_PREFIX,
        }
        sub_states = dict()
        for variable_reference in variable_references:
            if not candidate_sub_states:
                break
            for sub_states_key, sub_states_prefix in candidate_sub_states.items():
                if variable_reference.startswith(sub_states_prefix):
                    sub_states[sub_states_key] = self._states_data[sub_states_key]  # noqa
                    del candidate_sub_states[sub_states_key]
                    break
        return encode_jsonata_variable_declarations(
            bindings={_STATES_PREFIX: sub_states}
        )



import logging
from typing import Set

from moto.stepfunctions.parser.asl.component.state.exec.state_task.credentials import (
    ComputedCredentials,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_task.service.resource import (
    ResourceCondition,
    ResourceRuntimePart,
)
from moto.stepfunctions.parser.asl.component.state.exec.state_task.service.state_task_service_callback import (
    StateTaskServiceCallback,
)
from moto.stepfunctions.parser.asl.eval.environment import Environment
from moto.stepfunctions.parser.asl.utils.boto_client import boto_client_for

LOG = logging.getLogger(__name__)

_SUPPORTED_INTEGRATION_PATTERNS: Set[ResourceCondition] = {
    ResourceCondition.WaitForTaskToken,
}


class StateTaskServiceUnsupported(StateTaskServiceCallback):
    def __init__(self):
        super().__init__(supported_integration_patterns=_SUPPORTED_INTEGRATION_PATTERNS)

    def _log_unsupported_warning(self):
        # Logs that the optimised service integration is not supported,
        # however the request is being forwarded to the service.
        service_name = self._get_boto_service_name()
        resource_arn = self.resource.resource_arn
        LOG.warning(
            "Unsupported Optimised service integration for service_name '%s' in resource: '%s'. "
            "Attempting to forward request to service.",
            service_name,
            resource_arn,
        )

    def _eval_service_task(
        self,
        env: Environment,
        resource_runtime_part: ResourceRuntimePart,
        normalised_parameters: dict,
        task_credentials: ComputedCredentials,
    ):
        # Logs that the evaluation of this optimised service integration is not supported
        # and relays the call to the target service with the computed parameters.
        self._log_unsupported_warning()
        service_name = self._get_boto_service_name()
        boto_action = self._get_boto_service_action()
        boto_client = boto_client_for(
            region=resource_runtime_part.region,
            account=resource_runtime_part.account,
            service=service_name,
            credentials=task_credentials,
        )
        response = getattr(boto_client, boto_action)(**normalised_parameters)
        response.pop("ResponseMetadata", None)
        env.stack.append(response)



import json
import re

from jsonpath_ng.ext import parse
from jsonpath_ng.jsonpath import Index

from moto.stepfunctions.parser.asl.utils.encoding import to_json_str

_PATTERN_SINGLETON_ARRAY_ACCESS_OUTPUT: str = r"\[\d+\$"


def _is_singleton_array_access(path: str) -> bool:
    # Returns true if the json path terminates with a literal singleton array access.
    return bool(re.search(_PATTERN_SINGLETON_ARRAY_ACCESS_OUTPUT, path))


def extract_json(path: str, data: json) -> json:
    input_expr = parse(path)

    matches = input_expr.find(data)
    if not matches:
        raise RuntimeError(
            f"The JSONPath {path} could not be found in the input {to_json_str(data)}"
        )

    if (
        len(matches) > 1
        or isinstance(matches[0].path, Index)
        or (matches[0].context and isinstance(matches[0].context.path, Index))
    ):
        value = [match.value for match in matches]

        # AWS StepFunctions breaks jsonpath specifications and instead
        # unpacks literal singleton array accesses.
        if _is_singleton_array_access(path=path) and len(value) == 1:
            value = value[0]
    else:
        value = matches[0].value

    return value



from unittest import SkipTest

import requests

from moto import settings
from moto.server import ThreadedMotoServer

SERVER_PORT = 5001
BASE_URL = f"http://localhost:{SERVER_PORT}/"


class TestAccountIdResolution:
    def setup_method(self):
        if not settings.TEST_DECORATOR_MODE:
            raise SkipTest(
                "No point in testing this in ServerMode, as we already start our own server"
            )
        self.server = ThreadedMotoServer(port=SERVER_PORT, verbose=False)
        self.server.start()

    def teardown_method(self):
        self.server.stop()

    def test_with_custom_request_header(self):
        buckets_for_account_1 = ["foo", "bar"]
        for name in buckets_for_account_1:
            requests.put(f"http://{name}.localhost:{SERVER_PORT}/")

        res = requests.get(BASE_URL)
        assert b"<Name>foo</Name>" in res.content
        assert b"<Name>bar</Name>" in res.content

        # Create two more buckets in another account
        headers = {"x-moto-account-id": "333344445555"}
        buckets_for_account_2 = ["baz", "bla"]
        for name in buckets_for_account_2:
            requests.put(f"http://{name}.localhost:{SERVER_PORT}/", headers=headers)

        # Verify only these buckets exist in this account
        res = requests.get(BASE_URL, headers=headers)
        assert b"<Name>baz</Name>" in res.content
        assert b"<Name>bla</Name>" in res.content
        assert b"<Name>foo</Name>" not in res.content
        assert b"<Name>bar</Name>" not in res.content

        # Verify these buckets do not exist in the original account
        res = requests.get(BASE_URL)
        assert b"<Name>baz</Name>" not in res.content
        assert b"<Name>bla</Name>" not in res.content



from typing import Final, List

from moto.stepfunctions.parser.asl.component.component import Component


class CSVHeaders(Component):
    header_names: Final[List[str]]

    def __init__(self, header_names: List[str]):
        self.header_names = header_names



import re

from moto.stepfunctions.parser.asl.component.intrinsic.argument.function_argument_list import (
    FunctionArgumentList,
)
from moto.stepfunctions.parser.asl.component.intrinsic.function.statesfunction.states_function import (
    StatesFunction,
)
from moto.stepfunctions.parser.asl.component.intrinsic.functionname.state_function_name_types import (
    StatesFunctionNameType,
)
from moto.stepfunctions.parser.asl.component.intrinsic.functionname.states_function_name import (
    StatesFunctionName,
)
from moto.stepfunctions.parser.asl.eval.environment import Environment


class StringSplit(StatesFunction):
    # Splits a string into an array of values.
    #
    # For example:
    # With input
    # {
    #   "inputString": "This.is+a,test=string",
    #   "splitter": ".+,="
    # }
    #
    # The call
    # {
    #   "myStringArray.$": "States.StringSplit($.inputString, $.splitter)"
    # }
    #
    # Returns
    # {"myStringArray": [
    #   "This",
    #   "is",
    #   "a",
    #   "test",
    #   "string"
    # ]}
    def __init__(self, arg_list: FunctionArgumentList):
        super().__init__(
            states_name=StatesFunctionName(
                function_type=StatesFunctionNameType.StringSplit
            ),
            arg_list=arg_list,
        )
        if arg_list.size != 2:
            raise ValueError(
                f"Expected 2 arguments for function type '{type(self)}', but got: '{arg_list}'."
            )

    def _eval_body(self, env: Environment) -> None:
        self.arg_list.eval(env=env)
        args = env.stack.pop()

        del_chars = args.pop()
        if not isinstance(del_chars, str):
            raise ValueError(
                f"Expected string value as delimiting characters, but got '{del_chars}'."
            )

        string = args.pop()
        if not isinstance(del_chars, str):
            raise ValueError(f"Expected string value, but got '{del_chars}'.")

        pattern = "|".join(re.escape(c) for c in del_chars)

        parts = re.split(pattern, string)
        parts_clean = list(filter(bool, parts))
        env.stack.append(parts_clean)



"""DirectConnectBackend class with methods for supported APIs."""

from dataclasses import dataclass, field
from datetime import datetime
from typing import Any, Dict, List, Optional, Tuple

from moto.core.base_backend import BackendDict, BaseBackend
from moto.core.common_models import BaseModel

from .enums import (
    ConnectionStateType,
    EncryptionModeType,
    LagStateType,
    MacSecKeyStateType,
    PortEncryptionStatusType,
)
from .exceptions import (
    ConnectionIdMissing,
    ConnectionNotFound,
    LAGNotFound,
    MacSecKeyNotFound,
)


@dataclass
class MacSecKey(BaseModel):
    secret_arn: Optional[str]
    ckn: Optional[str]
    state: MacSecKeyStateType
    start_on: str
    cak: Optional[str] = None

    def to_dict(self) -> Dict[str, str]:
        return {
            "secretARN": self.secret_arn or "",
            "ckn": self.ckn or "",
            "state": self.state,
            "startOn": self.start_on,
        }


@dataclass
class Connection(BaseModel):
    aws_device_v2: str
    aws_device: str
    aws_logical_device_id: str
    bandwidth: str
    connection_name: str
    connection_state: ConnectionStateType
    encryption_mode: EncryptionModeType
    has_logical_redundancy: bool
    jumbo_frame_capable: bool
    lag_id: Optional[str]
    loa_issue_time: str
    location: str
    mac_sec_capable: Optional[bool]
    mac_sec_keys: List[MacSecKey]
    owner_account: str
    partner_name: str
    port_encryption_status: PortEncryptionStatusType
    provider_name: Optional[str]
    region: str
    tags: Optional[List[Dict[str, str]]]
    vlan: int
    connection_id: str = field(default="", init=False)

    def __post_init__(self) -> None:
        if self.connection_id == "":
            self.connection_id = f"dx-moto-{self.connection_name}-{datetime.now().strftime('%Y%m%d%H%M%S')}"

    def to_dict(
        self,
    ) -> Dict[str, Any]:
        return {
            "awsDevice": self.aws_device,
            "awsDeviceV2": self.aws_device_v2,
            "awsLogicalDeviceId": self.aws_logical_device_id,
            "bandwidth": self.bandwidth,
            "connectionId": self.connection_id,
            "connectionName": self.connection_name,
            "connectionState": self.connection_state,
            "encryptionMode": self.encryption_mode,
            "hasLogicalRedundancy": self.has_logical_redundancy,
            "jumboFrameCapable": self.jumbo_frame_capable,
            "lagId": self.lag_id,
            "loaIssueTime": self.loa_issue_time,
            "location": self.location,
            "macSecCapable": self.mac_sec_capable,
            "macSecKeys": [key.to_dict() for key in self.mac_sec_keys],
            "partnerName": self.partner_name,
            "portEncryptionStatus": self.port_encryption_status,
            "providerName": self.provider_name,
            "region": self.region,
            "tags": self.tags,
            "vlan": self.vlan,
        }


@dataclass
class LAG(BaseModel):
    aws_device_v2: str
    aws_device: str
    aws_logical_device_id: str
    connections_bandwidth: str
    number_of_connections: int
    minimum_links: int
    connections: List[Connection]
    lag_name: str
    lag_state: LagStateType
    encryption_mode: EncryptionModeType
    has_logical_redundancy: bool
    jumbo_frame_capable: bool
    location: str
    mac_sec_capable: Optional[bool]
    mac_sec_keys: List[MacSecKey]
    owner_account: str
    provider_name: Optional[str]
    region: str
    tags: Optional[List[Dict[str, str]]]
    lag_id: str = field(default="", init=False)

    def __post_init__(self) -> None:
        if self.lag_id == "":
            self.lag_id = (
                f"dxlag-moto-{self.lag_name}-{datetime.now().strftime('%Y%m%d%H%M%S')}"
            )

    def to_dict(
        self,
    ) -> Dict[str, Any]:
        return {
            "awsDevice": self.aws_device,
            "awsDeviceV2": self.aws_device_v2,
            "awsLogicalDeviceId": self.aws_logical_device_id,
            "connectionsBandwidth": self.connections_bandwidth,
            "numberOfConnections": self.number_of_connections,
            "minimumLinks": self.minimum_links,
            "connections": [conn.to_dict() for conn in self.connections],
            "lagId": self.lag_id,
            "lagName": self.lag_name,
            "lagState": self.lag_state,
            "encryptionMode": self.encryption_mode,
            "hasLogicalRedundancy": self.has_logical_redundancy,
            "jumboFrameCapable": self.jumbo_frame_capable,
            "location": self.location,
            "macSecCapable": self.mac_sec_capable,
            "macSecKeys": [key.to_dict() for key in self.mac_sec_keys],
            "providerName": self.provider_name,
            "region": self.region,
            "tags": self.tags,
        }


class DirectConnectBackend(BaseBackend):
    """Implementation of DirectConnect APIs."""

    def __init__(self, region_name: str, account_id: str) -> None:
        super().__init__(region_name, account_id)
        self.connections: Dict[str, Connection] = {}
        self.lags: Dict[str, LAG] = {}

    def describe_connections(self, connection_id: Optional[str]) -> List[Connection]:
        if connection_id and connection_id not in self.connections:
            raise ConnectionNotFound(connection_id, self.region_name)
        if connection_id:
            connection = self.connections.get(connection_id)
            return [] if not connection else [connection]
        return list(self.connections.values())

    def create_connection(
        self,
        location: str,
        bandwidth: str,
        connection_name: str,
        lag_id: Optional[str],
        tags: Optional[List[Dict[str, str]]],
        provider_name: Optional[str],
        request_mac_sec: Optional[bool],
    ) -> Connection:
        encryption_mode = EncryptionModeType.NO
        mac_sec_keys = []
        if request_mac_sec:
            encryption_mode = EncryptionModeType.MUST
            mac_sec_keys = [
                MacSecKey(
                    secret_arn="mock_secret_arn",
                    ckn="mock_ckn",
                    state=MacSecKeyStateType.ASSOCIATED,
                    start_on=datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                )
            ]
        connection = Connection(
            aws_device_v2="mock_device_v2",
            aws_device="mock_device",
            aws_logical_device_id="mock_logical_device_id",
            bandwidth=bandwidth,
            connection_name=connection_name,
            connection_state=ConnectionStateType.AVAILABLE,
            encryption_mode=encryption_mode,
            has_logical_redundancy=False,
            jumbo_frame_capable=False,
            lag_id=lag_id,
            loa_issue_time=datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
            location=location,
            mac_sec_capable=request_mac_sec,
            mac_sec_keys=mac_sec_keys,
            owner_account=self.account_id,
            partner_name="mock_partner",
            port_encryption_status=PortEncryptionStatusType.DOWN,
            provider_name=provider_name,
            region=self.region_name,
            tags=tags,
            vlan=0,
        )
        self.connections[connection.connection_id] = connection
        return connection

    def delete_connection(self, connection_id: str) -> Connection:
        if not connection_id:
            raise ConnectionIdMissing()
        connection = self.connections.get(connection_id)
        if connection:
            self.connections[
                connection_id
            ].connection_state = ConnectionStateType.DELETED
            return connection
        raise ConnectionNotFound(connection_id, self.region_name)

    def update_connection(
        self,
        connection_id: str,
        new_connection_name: Optional[str],
        new_encryption_mode: Optional[EncryptionModeType],
    ) -> Connection:
        if not connection_id:
            raise ConnectionIdMissing()
        connection = self.connections.get(connection_id)
        if connection:
            if new_connection_name:
                self.connections[connection_id].connection_name = new_connection_name
            if new_encryption_mode:
                self.connections[connection_id].encryption_mode = new_encryption_mode
            return connection
        raise ConnectionNotFound(connection_id, self.region_name)

    def associate_mac_sec_key(
        self,
        connection_id: str,
        secret_arn: Optional[str],
        ckn: Optional[str],
        cak: Optional[str],
    ) -> Tuple[str, List[MacSecKey]]:
        mac_sec_key = MacSecKey(
            secret_arn=secret_arn or "mock_secret_arn",
            ckn=ckn,
            cak=cak,
            state=MacSecKeyStateType.ASSOCIATED,
            start_on=datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        )

        if connection_id.startswith("dxlag-"):
            return self._associate_mac_sec_key_with_lag(
                lag_id=connection_id, mac_sec_key=mac_sec_key
            )

        return self._associate_mac_sec_key_with_connection(
            connection_id=connection_id, mac_sec_key=mac_sec_key
        )

    def _associate_mac_sec_key_with_lag(
        self, lag_id: str, mac_sec_key: MacSecKey
    ) -> Tuple[str, List[MacSecKey]]:
        lag = self.lags.get(lag_id) or None
        if not lag:
            raise LAGNotFound(lag_id, self.region_name)

        lag.mac_sec_keys.append(mac_sec_key)
        for connection in lag.connections:
            connection.mac_sec_keys = lag.mac_sec_keys

        return lag_id, lag.mac_sec_keys

    def _associate_mac_sec_key_with_connection(
        self, connection_id: str, mac_sec_key: MacSecKey
    ) -> Tuple[str, List[MacSecKey]]:
        connection = self.connections.get(connection_id) or None
        if not connection:
            raise ConnectionNotFound(connection_id, self.region_name)

        self.connections[connection_id].mac_sec_keys.append(mac_sec_key)
        return connection_id, self.connections[connection_id].mac_sec_keys

    def create_lag(
        self,
        number_of_connections: int,
        location: str,
        connections_bandwidth: str,
        lag_name: str,
        connection_id: Optional[str],
        tags: Optional[List[Dict[str, str]]],
        child_connection_tags: Optional[List[Dict[str, str]]],
        provider_name: Optional[str],
        request_mac_sec: Optional[bool],
    ) -> LAG:
        if connection_id:
            raise NotImplementedError(
                "creating a lag with a connection_id is not currently supported by moto"
            )
        encryption_mode = EncryptionModeType.NO
        mac_sec_keys = []
        if request_mac_sec:
            encryption_mode = EncryptionModeType.MUST
            mac_sec_keys = [
                MacSecKey(
                    secret_arn="mock_secret_arn",
                    ckn="mock_ckn",
                    state=MacSecKeyStateType.ASSOCIATED,
                    start_on=datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
                )
            ]
        lag = LAG(
            aws_device_v2="mock_device_v2",
            aws_device="mock_device",
            aws_logical_device_id="mock_logical_device_id",
            connections_bandwidth=connections_bandwidth,
            lag_name=lag_name,
            lag_state=LagStateType.AVAILABLE,
            minimum_links=0,
            encryption_mode=encryption_mode,
            has_logical_redundancy=False,
            jumbo_frame_capable=False,
            number_of_connections=number_of_connections,
            connections=[],
            location=location,
            mac_sec_capable=request_mac_sec,
            mac_sec_keys=mac_sec_keys,
            owner_account=self.account_id,
            provider_name=provider_name,
            region=self.region_name,
            tags=tags,
        )
        for i in range(number_of_connections):
            connection = self.create_connection(
                location=location,
                bandwidth=connections_bandwidth,
                connection_name=f"Requested Connection {i+1} for Lag {lag.lag_id}",
                lag_id=lag.lag_id,
                tags=child_connection_tags,
                request_mac_sec=False,
                provider_name=provider_name,
            )
            if request_mac_sec:
                connection.mac_sec_capable = True
                connection.mac_sec_keys = mac_sec_keys
                connection.encryption_mode = encryption_mode
            lag.connections.append(connection)

        self.lags[lag.lag_id] = lag
        return lag

    def describe_lags(self, lag_id: Optional[str]) -> List[LAG]:
        if lag_id and lag_id not in self.lags:
            raise LAGNotFound(lag_id, self.region_name)
        if lag_id:
            lag = self.lags.get(lag_id)
            return [] if not lag else [lag]
        return list(self.lags.values())

    def disassociate_mac_sec_key(
        self, connection_id: str, secret_arn: str
    ) -> Tuple[str, MacSecKey]:
        mac_sec_keys: List[MacSecKey] = []
        if connection_id.startswith("dxlag-"):
            if connection_id in self.lags:
                mac_sec_keys = self.lags[connection_id].mac_sec_keys
        elif connection_id in self.connections:
            mac_sec_keys = self.connections[connection_id].mac_sec_keys
        if not mac_sec_keys:
            raise ConnectionNotFound(connection_id, self.region_name)

        arn_casefold = secret_arn.casefold()
        for i, mac_sec_key in enumerate(mac_sec_keys):
            if str(mac_sec_key.secret_arn).casefold() == arn_casefold:
                mac_sec_key.state = MacSecKeyStateType.DISASSOCIATED
                return connection_id, mac_sec_keys.pop(i)

        raise MacSecKeyNotFound(secret_arn=secret_arn, connection_id=connection_id)


directconnect_backends = BackendDict(DirectConnectBackend, "directconnect")



"""Exceptions raised by the networkmanager service."""

import json

from moto.core.exceptions import JsonRESTError


class ValidationError(JsonRESTError):
    def __init__(self, message: str):
        super().__init__("ValidationException", message)


class ResourceNotFound(JsonRESTError):
    def __init__(self, resource_id: str):
        super().__init__("NotFoundException", "Resource not found.")
        body = {
            "ResourceId": resource_id,
            "Message": "Resource not found.",
        }
        self.description = json.dumps(body)



from moto.stepfunctions.parser.asl.component.common.variable_sample import (
    VariableSample,
)
from moto.stepfunctions.parser.asl.component.intrinsic.argument.function_argument import (
    FunctionArgument,
)
from moto.stepfunctions.parser.asl.eval.environment import Environment


class FunctionArgumentVar(FunctionArgument):
    variable_sample: VariableSample

    def __init__(self, variable_sample: VariableSample):
        super().__init__()
        self.variable_sample = variable_sample

    def _eval_body(self, env: Environment) -> None:
        self.variable_sample.eval(env=env)
        self._value = env.stack.pop()
        super()._eval_body(env=env)



"""Unit tests for shield-supported APIs."""

import boto3
import pytest
from botocore.exceptions import ClientError

from moto import mock_aws

# See our Development Tips on writing tests for hints on how to write good tests:
# http://docs.getmoto.org/en/latest/docs/contributing/development_tips/tests.html


@mock_aws
def test_create_protection():
    client = boto3.client("shield")
    resp = client.create_protection(
        Name="foobar",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
        Tags=[
            {"Key": "key1", "Value": "value1"},
        ],
    )
    assert "ProtectionId" in resp


@mock_aws
def test_create_protection_resource_already_exists():
    client = boto3.client("shield")
    client.create_protection(
        Name="foobar",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
    )
    with pytest.raises(ClientError) as exc:
        client.create_protection(
            Name="foobar",
            ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
        )
    err = exc.value.response["Error"]
    assert err["Code"] == "ResourceAlreadyExistsException"


@mock_aws
def test_create_protection_invalid_resource():
    client = boto3.client("shield")
    with pytest.raises(ClientError) as exc:
        client.create_protection(
            Name="foobar",
            ResourceArn="arn:aws:dynamodb:us-east-1:123456789012:table/foobar",
        )
    err = exc.value.response["Error"]
    assert err["Code"] == "InvalidResourceException"
    assert err["Message"] == "Unrecognized resource 'table' of service 'dynamodb'."

    with pytest.raises(ClientError) as exc:
        client.create_protection(
            Name="foobar",
            ResourceArn="arn:aws:sns:us-east-2:123456789012:MyTopic",
        )
    err = exc.value.response["Error"]
    assert err["Code"] == "InvalidResourceException"
    assert err["Message"] == "Relative ID must be in the form '<resource>/<id>'."


@mock_aws
def test_describe_protection_with_resource_arn():
    client = boto3.client("shield")
    client.create_protection(
        Name="foobar",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
    )
    resp = client.describe_protection(
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar"
    )
    protection = resp["Protection"]
    assert "Id" in protection
    assert "Name" in protection
    assert "ResourceArn" in protection
    assert "ProtectionArn" in protection


@mock_aws
def test_describe_protection_with_protection_id():
    client = boto3.client("shield")
    protection = client.create_protection(
        Name="foobar",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
    )
    protection_id = protection["ProtectionId"]
    resp = client.describe_protection(ProtectionId=protection_id)
    protection = resp["Protection"]
    assert "Id" in protection
    assert "Name" in protection
    assert "ResourceArn" in protection
    assert "ProtectionArn" in protection


@mock_aws
def test_describe_protection_with_both_resource_and_protection_id():
    client = boto3.client("shield")
    with pytest.raises(ClientError) as exc:
        client.describe_protection(
            ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
            ProtectionId="aaaaaaaa-bbbb-cccc-dddd-aaa221177777",
        )
    err = exc.value.response["Error"]
    assert err["Code"] == "InvalidParameterException"


@mock_aws
def test_describe_protection_resource_doesnot_exist():
    client = boto3.client("shield")
    with pytest.raises(ClientError) as exc:
        client.describe_protection(
            ResourceArn="arn:aws:cloudfront::123456789012:distribution/donotexist"
        )
    err = exc.value.response["Error"]
    assert err["Code"] == "ResourceNotFoundException"


@mock_aws
def test_describe_protection_doesnot_exist():
    client = boto3.client("shield")
    with pytest.raises(ClientError) as exc:
        client.describe_protection(ProtectionId="aaaaaaaa-bbbb-cccc-dddd-aaa221177777")
    err = exc.value.response["Error"]
    assert err["Code"] == "ResourceNotFoundException"


@mock_aws
def test_list_protections():
    client = boto3.client("shield")
    client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
    )
    client.create_protection(
        Name="shield2",
        ResourceArn="arn:aws:elasticloadbalancing:us-east-1:123456789012:loadbalancer/foobar",
    )
    resp = client.list_protections()
    assert "Protections" in resp
    assert len(resp["Protections"]) == 2


@mock_aws
def test_list_protections_with_only_resource_arn():
    client = boto3.client("shield")
    client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
    )
    client.create_protection(
        Name="shield2",
        ResourceArn="arn:aws:elasticloadbalancing:us-east-1:123456789012:loadbalancer/foobar",
    )
    resp = client.list_protections(
        InclusionFilters={
            "ResourceArns": ["arn:aws:cloudfront::123456789012:distribution/foobar"],
        }
    )
    assert "Protections" in resp
    assert len(resp["Protections"]) == 1


@mock_aws
def test_list_protections_with_only_protection_name():
    client = boto3.client("shield")
    client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
    )
    client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:elasticloadbalancing:us-east-1:123456789012:loadbalancer/foobar",
    )
    resp = client.list_protections(
        InclusionFilters={
            "ProtectionNames": ["shield1"],
        }
    )
    assert "Protections" in resp
    assert len(resp["Protections"]) == 2


@mock_aws
def test_list_protections_with_only_resource_type():
    client = boto3.client("shield")
    client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
    )
    client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:elasticloadbalancing:us-east-1:123456789012:loadbalancer/foobar",
    )
    client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:elasticloadbalancing:us-east-1:123456789012:loadbalancer/app/my-load-balancer/1234567890123456",
    )
    resp_elb = client.list_protections(
        InclusionFilters={
            "ResourceTypes": ["CLASSIC_LOAD_BALANCER"],
        }
    )
    assert "Protections" in resp_elb
    assert len(resp_elb["Protections"]) == 1
    resp_alb = client.list_protections(
        InclusionFilters={
            "ResourceTypes": ["APPLICATION_LOAD_BALANCER"],
        }
    )
    assert len(resp_alb["Protections"]) == 1


@mock_aws
def test_list_protections_with_resource_arn_and_protection_name():
    client = boto3.client("shield")
    client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
    )
    client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:elasticloadbalancing:us-east-1:123456789012:loadbalancer/foobar",
    )
    resp = client.list_protections(
        InclusionFilters={
            "ResourceArns": ["arn:aws:cloudfront::123456789012:distribution/foobar"],
            "ProtectionNames": ["shield1"],
        }
    )
    assert "Protections" in resp
    assert len(resp["Protections"]) == 1


@mock_aws
def test_list_protections_invalid_resource_arn():
    client = boto3.client("shield")
    with pytest.raises(ClientError) as exc:
        client.list_protections(
            InclusionFilters={
                "ResourceArns": [
                    "arn:aws:cloudfront::123456789012:distribution/foobar",
                    "arn:aws:cloudfront::123456789012:distribution/foobar2",
                ],
            }
        )
    err = exc.value.response["Error"]
    assert err["Code"] == "ValidationException"


@mock_aws
def test_list_protections_invalid_protection_names():
    client = boto3.client("shield")
    with pytest.raises(ClientError) as exc:
        client.list_protections(
            InclusionFilters={
                "ProtectionNames": ["shield1", "shield2"],
            }
        )
    err = exc.value.response["Error"]
    assert err["Code"] == "ValidationException"


@mock_aws
def test_list_protections_invalid_resource_types():
    client = boto3.client("shield")
    with pytest.raises(ClientError) as exc:
        client.list_protections(
            InclusionFilters={
                "ResourceTypes": ["CLOUDFRONT_DISTRIBUTION", "ROUTE_53_HOSTED_ZONE"],
            }
        )
    err = exc.value.response["Error"]
    assert err["Code"] == "ValidationException"


@mock_aws
def test_delete_protection():
    client = boto3.client("shield")
    protection = client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:elasticloadbalancing:us-east-1:123456789012:loadbalancer/foobar",
    )
    client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
    )
    client.delete_protection(ProtectionId=protection["ProtectionId"])
    with pytest.raises(ClientError) as exc:
        client.describe_protection(ProtectionId=protection["ProtectionId"])
    err = exc.value.response["Error"]
    assert err["Code"] == "ResourceNotFoundException"
    resp = client.list_protections()
    assert len(resp["Protections"]) == 1


@mock_aws
def test_delete_protection_invalid_protection_id():
    client = boto3.client("shield")
    with pytest.raises(ClientError) as exc:
        client.delete_protection(ProtectionId="aaaaaaaa-bbbb-cccc-dddd-aaa221177777")
    err = exc.value.response["Error"]
    assert err["Code"] == "ResourceNotFoundException"


@mock_aws
def test_list_tags_for_resource():
    client = boto3.client("shield")
    protection = client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
        Tags=[
            {"Key": "key1", "Value": "value1"},
            {"Key": "key2", "Value": "value2"},
        ],
    )
    desc_protection = client.describe_protection(
        ProtectionId=protection["ProtectionId"]
    )
    resp = client.list_tags_for_resource(
        ResourceARN=desc_protection["Protection"]["ProtectionArn"]
    )
    assert len(resp["Tags"]) == 2


@mock_aws
def test_tag_resource():
    client = boto3.client("shield")
    protection = client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
        Tags=[{"Key": "key1", "Value": "value1"}],
    )
    desc_protection = client.describe_protection(
        ProtectionId=protection["ProtectionId"]
    )
    protection_arn = desc_protection["Protection"]["ProtectionArn"]
    client.tag_resource(
        ResourceARN=protection_arn,
        Tags=[
            {"Key": "key2", "Value": "value2"},
        ],
    )
    resp = client.list_tags_for_resource(ResourceARN=protection_arn)
    assert len(resp["Tags"]) == 2


@mock_aws
def test_untag_resource():
    client = boto3.client("shield")
    protection = client.create_protection(
        Name="shield1",
        ResourceArn="arn:aws:cloudfront::123456789012:distribution/foobar",
        Tags=[
            {"Key": "key1", "Value": "value1"},
            {"Key": "key2", "Value": "value2"},
        ],
    )
    desc_protection = client.describe_protection(
        ProtectionId=protection["ProtectionId"]
    )
    protection_arn = desc_protection["Protection"]["ProtectionArn"]
    client.untag_resource(
        ResourceARN=protection_arn,
        TagKeys=[
            "key1",
        ],
    )
    resp = client.list_tags_for_resource(ResourceARN=protection_arn)
    assert len(resp["Tags"]) == 1
    assert "key2" == resp["Tags"][0]["Key"]


@mock_aws
def test_create_and_describe_subscription():
    client = boto3.client("shield", region_name="eu-west-1")

    # Check exception when subscription does not exist
    with pytest.raises(ClientError) as exc:
        connection = client.describe_subscription()
    err = exc.value.response["Error"]
    assert err["Code"] == "ResourceNotFoundException"
    assert err["Message"] == "The subscription does not exist."

    client.create_subscription()
    connection = client.describe_subscription()
    subscription = connection["Subscription"]
    assert subscription["AutoRenew"] == "ENABLED"
    assert subscription["Limits"][0]["Type"] == "MitigationCapacityUnits"
    assert subscription["Limits"][0]["Max"] == 10000
    assert subscription["ProactiveEngagementStatus"] == "ENABLED"
    assert (
        subscription["SubscriptionLimits"]["ProtectionLimits"][
            "ProtectedResourceTypeLimits"
        ][0]["Type"]
        == "ELASTIC_IP_ADDRESS"
    )
    assert (
        subscription["SubscriptionLimits"]["ProtectionLimits"][
            "ProtectedResourceTypeLimits"
        ][0]["Max"]
        == 100
    )
    assert (
        subscription["SubscriptionLimits"]["ProtectionLimits"][
            "ProtectedResourceTypeLimits"
        ][1]["Type"]
        == "APPLICATION_LOAD_BALANCER"
    )
    assert (
        subscription["SubscriptionLimits"]["ProtectionLimits"][
            "ProtectedResourceTypeLimits"
        ][1]["Max"]
        == 50
    )
    assert (
        subscription["SubscriptionLimits"]["ProtectionGroupLimits"][
            "MaxProtectionGroups"
        ]
        == 20
    )
    assert (
        subscription["SubscriptionLimits"]["ProtectionGroupLimits"][
            "PatternTypeLimits"
        ]["ArbitraryPatternLimits"]["MaxMembers"]
        == 100
    )
    assert subscription["TimeCommitmentInSeconds"] == 31536000



import json
from uuid import uuid4

from moto.core.responses import BaseResponse
from moto.core.utils import unix_time

from .models import SSOAdminBackend, ssoadmin_backends


class SSOAdminResponse(BaseResponse):
    """Handler for SSOAdmin requests and responses."""

    def __init__(self) -> None:
        super().__init__(service_name="sso-admin")

    @property
    def ssoadmin_backend(self) -> SSOAdminBackend:
        """Return backend instance specific for this region."""
        return ssoadmin_backends[self.current_account][self.region]

    def create_account_assignment(self) -> str:
        params = json.loads(self.body)
        instance_arn = params.get("InstanceArn")
        target_id = params.get("TargetId")
        target_type = params.get("TargetType")
        permission_set_arn = params.get("PermissionSetArn")
        principal_type = params.get("PrincipalType")
        principal_id = params.get("PrincipalId")
        summary = self.ssoadmin_backend.create_account_assignment(
            instance_arn=instance_arn,
            target_id=target_id,
            target_type=target_type,
            permission_set_arn=permission_set_arn,
            principal_type=principal_type,
            principal_id=principal_id,
        )
        summary["Status"] = "SUCCEEDED"
        return json.dumps({"AccountAssignmentCreationStatus": summary})

    def delete_account_assignment(self) -> str:
        params = json.loads(self.body)
        instance_arn = params.get("InstanceArn")
        target_id = params.get("TargetId")
        target_type = params.get("TargetType")
        permission_set_arn = params.get("PermissionSetArn")
        principal_type = params.get("PrincipalType")
        principal_id = params.get("PrincipalId")
        summary = self.ssoadmin_backend.delete_account_assignment(
            instance_arn=instance_arn,
            target_id=target_id,
            target_type=target_type,
            permission_set_arn=permission_set_arn,
            principal_type=principal_type,
            principal_id=principal_id,
        )
        summary["Status"] = "SUCCEEDED"
        return json.dumps({"AccountAssignmentDeletionStatus": summary})

    def list_account_assignments(self) -> str:
        params = json.loads(self.body)
        instance_arn = params.get("InstanceArn")
        account_id = params.get("AccountId")
        permission_set_arn = params.get("PermissionSetArn")
        max_results = self._get_param("MaxResults")
        next_token = self._get_param("NextToken")

        assignments, next_token = self.ssoadmin_backend.list_account_assignments(
            instance_arn=instance_arn,
            account_id=account_id,
            permission_set_arn=permission_set_arn,
            next_token=next_token,
            max_results=max_results,
        )

        return json.dumps(dict(AccountAssignments=assignments, NextToken=next_token))

    def list_account_assignments_for_principal(self) -> str:
        filter_ = self._get_param("Filter", {})
        instance_arn = self._get_param("InstanceArn")
        max_results = self._get_param("MaxResults")
        next_token = self._get_param("NextToken")
        principal_id = self._get_param("PrincipalId")
        principal_type = self._get_param("PrincipalType")

        (
            assignments,
            next_token,
        ) = self.ssoadmin_backend.list_account_assignments_for_principal(
            filter_=filter_,
            instance_arn=instance_arn,
            max_results=max_results,
            next_token=next_token,
            principal_id=principal_id,
            principal_type=principal_type,
        )

        return json.dumps(dict(AccountAssignments=assignments, NextToken=next_token))

    def create_permission_set(self) -> str:
        name = self._get_param("Name")
        description = self._get_param("Description")
        instance_arn = self._get_param("InstanceArn")
        session_duration = self._get_param("SessionDuration", 3600)
        relay_state = self._get_param("RelayState")
        tags = self._get_param("Tags")

        permission_set = self.ssoadmin_backend.create_permission_set(
            name=name,
            description=description,
            instance_arn=instance_arn,
            session_duration=session_duration,
            relay_state=relay_state,
            tags=tags,
        )

        return json.dumps({"PermissionSet": permission_set})

    def delete_permission_set(self) -> str:
        params = json.loads(self.body)
        instance_arn = params.get("InstanceArn")
        permission_set_arn = params.get("PermissionSetArn")
        self.ssoadmin_backend.delete_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
        )
        return "{}"

    def update_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")
        description = self._get_param("Description")
        session_duration = self._get_param("SessionDuration", 3600)
        relay_state = self._get_param("RelayState")

        self.ssoadmin_backend.update_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
            description=description,
            session_duration=session_duration,
            relay_state=relay_state,
        )
        return "{}"

    def describe_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")

        permission_set = self.ssoadmin_backend.describe_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
        )
        return json.dumps({"PermissionSet": permission_set})

    def list_permission_sets(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        max_results = self._get_int_param("MaxResults")
        next_token = self._get_param("NextToken")
        permission_sets, next_token = self.ssoadmin_backend.list_permission_sets(
            instance_arn=instance_arn, max_results=max_results, next_token=next_token
        )
        permission_set_ids = []
        for permission_set in permission_sets:
            permission_set_ids.append(permission_set.permission_set_arn)
        response = {"PermissionSets": permission_set_ids}
        if next_token:
            response["NextToken"] = next_token
        return json.dumps(response)

    def put_inline_policy_to_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")
        inline_policy = self._get_param("InlinePolicy")
        self.ssoadmin_backend.put_inline_policy_to_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
            inline_policy=inline_policy,
        )
        return json.dumps({})

    def get_inline_policy_for_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")
        inline_policy = self.ssoadmin_backend.get_inline_policy_for_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
        )
        return json.dumps({"InlinePolicy": inline_policy})

    def delete_inline_policy_from_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")
        self.ssoadmin_backend.delete_inline_policy_from_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
        )
        return json.dumps({})

    def attach_managed_policy_to_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")
        managed_policy_arn = self._get_param("ManagedPolicyArn")
        self.ssoadmin_backend.attach_managed_policy_to_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
            managed_policy_arn=managed_policy_arn,
        )
        return json.dumps({})

    def list_managed_policies_in_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")
        max_results = self._get_int_param("MaxResults")
        next_token = self._get_param("NextToken")

        (
            managed_policies,
            next_token,
        ) = self.ssoadmin_backend.list_managed_policies_in_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
            max_results=max_results,
            next_token=next_token,
        )

        managed_policies_response = [
            {"Arn": managed_policy.arn, "Name": managed_policy.name}
            for managed_policy in managed_policies
        ]
        return json.dumps(
            {
                "AttachedManagedPolicies": managed_policies_response,
                "NextToken": next_token,
            }
        )

    def detach_managed_policy_from_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")
        managed_policy_arn = self._get_param("ManagedPolicyArn")
        self.ssoadmin_backend.detach_managed_policy_from_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
            managed_policy_arn=managed_policy_arn,
        )
        return json.dumps({})

    def attach_customer_managed_policy_reference_to_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")
        customer_managed_policy_reference = self._get_param(
            "CustomerManagedPolicyReference"
        )
        self.ssoadmin_backend.attach_customer_managed_policy_reference_to_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
            customer_managed_policy_reference=customer_managed_policy_reference,
        )
        return json.dumps({})

    def list_customer_managed_policy_references_in_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")
        max_results = self._get_int_param("MaxResults")
        next_token = self._get_param("NextToken")

        (
            customer_managed_policy_references,
            next_token,
        ) = self.ssoadmin_backend.list_customer_managed_policy_references_in_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
            max_results=max_results,
            next_token=next_token,
        )

        customer_managed_policy_references_response = [
            {
                "Name": customer_managed_policy_reference.name,
                "Path": customer_managed_policy_reference.path,
            }
            for customer_managed_policy_reference in customer_managed_policy_references
        ]
        return json.dumps(
            {
                "CustomerManagedPolicyReferences": customer_managed_policy_references_response,
                "NextToken": next_token,
            }
        )

    def detach_customer_managed_policy_reference_from_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")
        customer_managed_policy_reference = self._get_param(
            "CustomerManagedPolicyReference"
        )
        self.ssoadmin_backend.detach_customer_managed_policy_reference_from_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
            customer_managed_policy_reference=customer_managed_policy_reference,
        )
        return json.dumps({})

    def describe_account_assignment_creation_status(self) -> str:
        account_assignment_creation_request_id = self._get_param(
            "AccountAssignmentCreationRequestId"
        )
        instance_arn = self._get_param("InstanceArn")
        account_assignment_creation_status = self.ssoadmin_backend.describe_account_assignment_creation_status(
            account_assignment_creation_request_id=account_assignment_creation_request_id,
            instance_arn=instance_arn,
        )
        account_assignment_creation_status["Status"] = "SUCCEEDED"
        return json.dumps(
            dict(AccountAssignmentCreationStatus=account_assignment_creation_status)
        )

    def describe_account_assignment_deletion_status(self) -> str:
        account_assignment_deletion_request_id = self._get_param(
            "AccountAssignmentDeletionRequestId"
        )
        instance_arn = self._get_param("InstanceArn")
        account_assignment_deletion_status = self.ssoadmin_backend.describe_account_assignment_deletion_status(
            account_assignment_deletion_request_id=account_assignment_deletion_request_id,
            instance_arn=instance_arn,
        )
        account_assignment_deletion_status["Status"] = "SUCCEEDED"
        return json.dumps(
            dict(AccountAssignmentDeletionStatus=account_assignment_deletion_status)
        )

    def list_instances(self) -> str:
        instances = self.ssoadmin_backend.list_instances()

        return json.dumps({"Instances": [i.to_json() for i in instances]})

    def update_instance(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        name = self._get_param("Name")

        self.ssoadmin_backend.update_instance(instance_arn=instance_arn, name=name)

        return "{}"

    def provision_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")

        self.ssoadmin_backend.provision_permission_set(
            instance_arn=instance_arn,
            permission_set_arn=permission_set_arn,
        )
        return json.dumps(
            {
                "PermissionSetProvisioningStatus": {
                    "AccountId": self.current_account,
                    "CreatedDate": unix_time(),
                    "PermissionSetArn": permission_set_arn,
                    "RequestId": str(uuid4()),
                    "Status": "SUCCEEDED",
                }
            }
        )

    def list_permission_sets_provisioned_to_account(self) -> str:
        instance_arn = self._get_param("InstanceArn")

        permission_sets = (
            self.ssoadmin_backend.list_permission_sets_provisioned_to_account(
                instance_arn
            )
        )
        arns = [p.permission_set_arn for p in permission_sets]
        return json.dumps({"PermissionSets": arns})

    def list_accounts_for_provisioned_permission_set(self) -> str:
        instance_arn = self._get_param("InstanceArn")
        permission_set_arn = self._get_param("PermissionSetArn")

        account_ids = (
            self.ssoadmin_backend.list_accounts_for_provisioned_permission_set(
                instance_arn=instance_arn, permission_set_arn=permission_set_arn
            )
        )
        return json.dumps({"AccountIds": account_ids})



from typing import Set

from moto.stepfunctions.parser.api import VariableName, VariableNameList
from moto.stepfunctions.parser.asl.antlr.runtime.ASLIntrinsicParser import (
    ASLIntrinsicParser,
)
from moto.stepfunctions.parser.asl.jsonata.jsonata import (
    VariableReference,
    extract_jsonata_variable_references,
)
from moto.stepfunctions.parser.asl.static_analyser.intrinsic.intrinsic_static_analyser import (
    IntrinsicStaticAnalyser,
)


class VariableNamesIntrinsicStaticAnalyser(IntrinsicStaticAnalyser):
    _variable_names: VariableNameList

    def __init__(self):
        super().__init__()
        self._variable_names = list()

    @staticmethod
    def process_and_get(definition: str) -> VariableNameList:
        analyser = VariableNamesIntrinsicStaticAnalyser()
        analyser.analyse(definition=definition)
        return analyser.get_variable_name_list()

    def get_variable_name_list(self) -> VariableNameList:
        return self._variable_names

    def visitFunc_arg_list(self, ctx: ASLIntrinsicParser.Func_arg_listContext) -> None:
        # TODO: the extraction logic is not always in the same order as AWS's
        for child in ctx.children[::-1]:
            self.visit(child)

    def visitFunc_arg_var(self, ctx: ASLIntrinsicParser.Func_arg_varContext) -> None:
        variable_references: Set[VariableReference] = (
            extract_jsonata_variable_references(ctx.STRING_VARIABLE().getText())
        )
        for variable_reference in variable_references:
            variable_name: VariableName = variable_reference[1:]
            self._variable_names.append(variable_name)



import json
from threading import Thread
from typing import TYPE_CHECKING, Any, Dict, Optional
from uuid import uuid4

from moto.core.utils import gzip_compress
from moto.utilities.utils import get_partition

if TYPE_CHECKING:
    from moto.dynamodb.models import Table
    from moto.s3.models import S3Backend


class TableExport(Thread):
    def __init__(
        self,
        s3_bucket: str,
        s3_prefix: str,
        region_name: str,
        account_id: str,
        table: "Table",
        export_format: str,
        export_type: str,
    ):
        super().__init__()
        self.partition = get_partition(region_name)
        self.table = table
        self.arn = f"arn:{self.partition}:dynamodb:{region_name}:{account_id}:table/{table.table_arn}/import/{str(uuid4()).replace('-', '')}"
        self.s3_bucket = s3_bucket
        self.s3_prefix = s3_prefix
        self.status = "IN_PROGRESS"
        self.export_format = export_format
        self.export_type = export_type
        self.account_id = account_id
        self.region_name = region_name

        self.failure_code: Optional[str] = None
        self.failure_message: Optional[str] = None
        self.item_count = 0
        self.processed_bytes = 0
        self.error_count = 0

    def run(self) -> None:
        try:
            from moto.s3.models import s3_backends

            s3_backend = s3_backends[self.account_id][self.partition]
            s3_backend.buckets[self.s3_bucket]

        except KeyError:
            self.status = "FAILED"
            self.failure_code = "S3NoSuchBucket"
            self.failure_message = "The specified bucket does not exist"
            return

        try:
            self._backup_to_s3_file(s3_backend)
        except Exception as e:
            self.status = "FAILED"
            self.failure_code = "UNKNOWN"
            self.failure_message = str(e)

    def _backup_to_s3_file(self, s3_backend: "S3Backend") -> None:
        backup = ""
        for item in self.table.all_items():
            json_item = item.to_json(root_attr_name="Item")
            backup += json.dumps(json_item) + "\n"
            self.processed_bytes += len(json_item)
            self.item_count += 1
        content = gzip_compress(backup.encode("utf-8"))
        s3_backend.put_object(
            bucket_name=self.s3_bucket,
            key_name=f"{self.s3_prefix}/AWSDynamoDB/{str(uuid4())}/data/{str(uuid4())}.gz",
            value=content,
        )

        self.status = "COMPLETED" if self.error_count == 0 else "FAILED"

    def response(self) -> Dict[str, Any]:
        return {
            "ExportArn": self.arn,
            "ExportStatus": self.status,
            "FailureCode": self.failure_code,
            "FailureMessage": self.failure_message,
            "ExportFormat": self.export_format,
            "ExportType": self.export_type,
            "ItemCount": self.item_count,
            "BilledSizeBytes": self.processed_bytes,
        }



import abc
import copy
from typing import Set

from moto.stepfunctions.parser.asl.component.common.jsonata.jsonata_template_value_terminal import (
    JSONataTemplateValueTerminalExpression,
)
from moto.stepfunctions.parser.asl.component.common.variable_sample import (
    VariableSample,
)
from moto.stepfunctions.parser.asl.component.eval_component import EvalComponent
from moto.stepfunctions.parser.asl.component.intrinsic.function.function import Function
from moto.stepfunctions.parser.asl.component.intrinsic.functionname.state_function_name_types import (
    StatesFunctionNameType,
)
from moto.stepfunctions.parser.asl.eval.environment import Environment
from moto.stepfunctions.parser.asl.parse.intrinsic.intrinsic_parser import (
    IntrinsicParser,
)
from moto.stepfunctions.parser.asl.utils.json_path import extract_json


class CauseDecl(EvalComponent, abc.ABC): ...


class CauseConst(CauseDecl):
    value: str

    def __init__(self, value: str):
        self.value = value

    def _eval_body(self, env: Environment) -> None:
        env.stack.append(self.value)


class CauseJSONata(CauseDecl):
    def __init__(
        self,
        jsonata_template_value_terminal_expression: JSONataTemplateValueTerminalExpression,
    ):
        super().__init__()
        self.jsonata_template_value_terminal_expression = (
            jsonata_template_value_terminal_expression
        )

    def _eval_body(self, env: Environment) -> None:
        self.jsonata_template_value_terminal_expression.eval(env=env)


class CauseVar(CauseDecl):
    variable_sample: VariableSample

    def __init__(self, variable_sample: VariableSample):
        self.variable_sample = variable_sample

    def _eval_body(self, env: Environment) -> None:
        self.variable_sample.eval(env=env)


_STRING_RETURN_FUNCTIONS: Set[str] = {
    typ.name()
    for typ in [
        StatesFunctionNameType.Format,
        StatesFunctionNameType.JsonToString,
        StatesFunctionNameType.ArrayGetItem,
        StatesFunctionNameType.Base64Decode,
        StatesFunctionNameType.Base64Encode,
        StatesFunctionNameType.Hash,
        StatesFunctionNameType.UUID,
    ]
}


class CausePathJsonPath(CauseConst):
    def _eval_body(self, env: Environment) -> None:
        current_output = env.stack[-1]
        cause = extract_json(self.value, current_output)
        env.stack.append(cause)


class CausePathContextObject(CauseConst):
    def _eval_body(self, env: Environment) -> None:
        value = extract_json(self.value, env.states.context_object.context_object_data)
        env.stack.append(copy.deepcopy(value))


class CausePathIntrinsicFunction(CauseConst):
    function: Function

    def __init__(self, value: str) -> None:
        super().__init__(value=value)
        self.function, _ = IntrinsicParser.parse(value)
        if self.function.name.name not in _STRING_RETURN_FUNCTIONS:
            raise ValueError(
                f"Unsupported Intrinsic Function for CausePath declaration: '{self.value}'."
            )

    def _eval_body(self, env: Environment) -> None:
        self.function.eval(env=env)



from ..apigatewayv2.urls import url_paths as url_paths_v2
from .responses import APIGatewayResponse

url_bases = [r"https?://apigateway\.(.+)\.amazonaws.com"]

url_paths = {
    "{0}/restapis$": APIGatewayResponse.dispatch,
    "{0}/restapis/$": APIGatewayResponse.get_rest_api_without_id,
    "{0}/restapis/(?P<function_id>[^/]+)/?$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/resources$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/authorizers$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/authorizers/(?P<authorizer_id>[^/]+)/?$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/stages$": APIGatewayResponse.dispatch,
    "{0}/tags/(?P<resourceArn>[^/]+)$": APIGatewayResponse.dispatch,
    "{0}/tags/arn:(?P<partition>[^/]+):apigateway:(?P<region_name>[^/]+)::/restapis/(?P<function_id>[^/]+)/stages/(?P<stage_name>[^/]+)$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/stages/(?P<stage_name>[^/]+)/?$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/stages/(?P<stage_name>[^/]+)/exports/(?P<export_type>[^/]+)/?$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/deployments$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/deployments/(?P<deployment_id>[^/]+)/?$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/resources/(?P<resource_id>[^/]+)/?$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/resources/(?P<resource_id>[^/]+)/methods/(?P<method_name>[^/]+)/?$": APIGatewayResponse.dispatch,
    r"{0}/restapis/(?P<function_id>[^/]+)/resources/(?P<resource_id>[^/]+)/methods/(?P<method_name>[^/]+)/responses/(?P<status_code>\d+)$": APIGatewayResponse.dispatch,
    r"{0}/restapis/(?P<function_id>[^/]+)/resources/(?P<resource_id>[^/]+)/methods/(?P<method_name>[^/]+)/integration$": APIGatewayResponse.dispatch,
    r"{0}/restapis/(?P<function_id>[^/]+)/resources/(?P<resource_id>[^/]+)/methods/(?P<method_name>[^/]+)/integration/responses/(?P<status_code>\d+)$": APIGatewayResponse.dispatch,
    r"{0}/restapis/(?P<function_id>[^/]+)/resources/(?P<resource_id>[^/]+)/methods/(?P<method_name>[^/]+)/integration/responses/(?P<status_code>\d+)/$": APIGatewayResponse.dispatch,
    "{0}/apikeys$": APIGatewayResponse.dispatch,
    "{0}/apikeys/(?P<apikey>[^/]+)": APIGatewayResponse.dispatch,
    "{0}/usageplans$": APIGatewayResponse.dispatch,
    "{0}/domainnames$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/models$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/models/(?P<model_name>[^/]+)/?$": APIGatewayResponse.dispatch,
    "{0}/domainnames/(?P<domain_name>[^/]+)/?$": APIGatewayResponse.dispatch,
    "{0}/domainnames/(?P<domain_name>[^/]+)/basepathmappings$": APIGatewayResponse.dispatch,
    "{0}/domainnames/(?P<domain_name>[^/]+)/basepathmappings/(?P<base_path_mapping>[^/]+)$": APIGatewayResponse.dispatch,
    "{0}/usageplans/(?P<usage_plan_id>[^/]+)/?$": APIGatewayResponse.dispatch,
    "{0}/usageplans/(?P<usage_plan_id>[^/]+)/keys$": APIGatewayResponse.dispatch,
    "{0}/usageplans/(?P<usage_plan_id>[^/]+)/keys/(?P<api_key_id>[^/]+)/?$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<function_id>[^/]+)/requestvalidators$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<api_id>[^/]+)/requestvalidators/(?P<validator_id>[^/]+)/?$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<api_id>[^/]+)/gatewayresponses/?$": APIGatewayResponse.dispatch,
    "{0}/restapis/(?P<api_id>[^/]+)/gatewayresponses/(?P<response_type>[^/]+)/?$": APIGatewayResponse.dispatch,
    "{0}/vpclinks$": APIGatewayResponse.dispatch,
    "{0}/vpclinks/(?P<vpclink_id>[^/]+)": APIGatewayResponse.dispatch,
    "{0}/account$": APIGatewayResponse.dispatch,
}

# Also manages the APIGatewayV2
url_paths.update(url_paths_v2)



import json
from typing import Any, Dict, List

from moto.core.responses import BaseResponse
from moto.secretsmanager.exceptions import (
    InvalidParameterException,
    InvalidRequestException,
    ValidationException,
)

from .models import SecretsManagerBackend, filter_keys, secretsmanager_backends


def _validate_filters(filters: List[Dict[str, Any]]) -> None:
    for idx, f in enumerate(filters):
        filter_key = f.get("Key", None)
        filter_values = f.get("Values", None)
        if filter_key is None:
            raise InvalidParameterException("Invalid filter key")
        if filter_key not in filter_keys():
            raise ValidationException(
                f"1 validation error detected: Value '{filter_key}' at 'filters.{(idx + 1)}.member.key' failed to satisfy constraint: "
                "Member must satisfy enum value set: [all, name, tag-key, description, tag-value]"
            )
        if filter_values is None:
            raise InvalidParameterException(
                f"Invalid filter values for key: {filter_key}"
            )


class SecretsManagerResponse(BaseResponse):
    def __init__(self) -> None:
        super().__init__(service_name="secretsmanager")

    @property
    def backend(self) -> SecretsManagerBackend:
        return secretsmanager_backends[self.current_account][self.region]

    def cancel_rotate_secret(self) -> str:
        secret_id = self._get_param("SecretId")
        return self.backend.cancel_rotate_secret(secret_id=secret_id)

    def get_secret_value(self) -> str:
        secret_id = self._get_param("SecretId")
        version_id = self._get_param("VersionId")
        version_stage = self._get_param("VersionStage")
        value = self.backend.get_secret_value(
            secret_id=secret_id, version_id=version_id, version_stage=version_stage
        )
        return json.dumps(value)

    def batch_get_secret_value(self) -> str:
        secret_id_list = self._get_param("SecretIdList", if_none=[])
        filters = self._get_param("Filters", if_none=[])
        max_results = self._get_int_param("MaxResults")
        next_token = self._get_param("NextToken")

        secret_values, errors, next_token = self.backend.batch_get_secret_value(
            secret_id_list=secret_id_list,
            filters=filters,
            max_results=max_results,
            next_token=next_token,
        )
        return json.dumps(
            dict(SecretValues=secret_values, Errors=errors, NextToken=next_token)
        )

    def create_secret(self) -> str:
        name = self._get_param("Name")
        secret_string = self._get_param("SecretString")
        secret_binary = self._get_param("SecretBinary")
        description = self._get_param("Description", if_none="")
        tags = self._get_param("Tags", if_none=[])
        kms_key_id = self._get_param("KmsKeyId")
        client_request_token = self._get_param("ClientRequestToken")
        replica_regions = self._get_param("AddReplicaRegions", [])
        force_overwrite = self._get_bool_param("ForceOverwriteReplicaSecret", False)

        return self.backend.create_secret(
            name=name,
            secret_string=secret_string,
            secret_binary=secret_binary,
            description=description,
            tags=tags,
            kms_key_id=kms_key_id,
            client_request_token=client_request_token,
            replica_regions=replica_regions,
            force_overwrite=force_overwrite,
        )

    def update_secret(self) -> str:
        secret_id = self._get_param("SecretId")
        secret_string = self._get_param("SecretString")
        secret_binary = self._get_param("SecretBinary")
        description = self._get_param("Description")
        client_request_token = self._get_param("ClientRequestToken")
        kms_key_id = self._get_param("KmsKeyId", if_none=None)
        return self.backend.update_secret(
            secret_id=secret_id,
            secret_string=secret_string,
            secret_binary=secret_binary,
            client_request_token=client_request_token,
            kms_key_id=kms_key_id,
            description=description,
        )

    def get_random_password(self) -> str:
        password_length = self._get_param("PasswordLength", if_none=32)
        exclude_characters = self._get_param("ExcludeCharacters", if_none="")
        exclude_numbers = self._get_param("ExcludeNumbers", if_none=False)
        exclude_punctuation = self._get_param("ExcludePunctuation", if_none=False)
        exclude_uppercase = self._get_param("ExcludeUppercase", if_none=False)
        exclude_lowercase = self._get_param("ExcludeLowercase", if_none=False)
        include_space = self._get_param("IncludeSpace", if_none=False)
        require_each_included_type = self._get_param(
            "RequireEachIncludedType", if_none=True
        )
        return self.backend.get_random_password(
            password_length=password_length,
            exclude_characters=exclude_characters,
            exclude_numbers=exclude_numbers,
            exclude_punctuation=exclude_punctuation,
            exclude_uppercase=exclude_uppercase,
            exclude_lowercase=exclude_lowercase,
            include_space=include_space,
            require_each_included_type=require_each_included_type,
        )

    def describe_secret(self) -> str:
        secret_id = self._get_param("SecretId")
        secret = self.backend.describe_secret(secret_id=secret_id)
        return json.dumps(secret.to_dict())

    def rotate_secret(self) -> str:
        client_request_token = self._get_param("ClientRequestToken")
        rotation_lambda_arn = self._get_param("RotationLambdaARN")
        rotation_rules = self._get_param("RotationRules")
        secret_id = self._get_param("SecretId")
        rotate_immediately = self._get_bool_param("RotateImmediately", True)
        return self.backend.rotate_secret(
            secret_id=secret_id,
            client_request_token=client_request_token,
            rotation_lambda_arn=rotation_lambda_arn,
            rotation_rules=rotation_rules,
            rotate_immediately=rotate_immediately,
        )

    def put_secret_value(self) -> str:
        secret_id = self._get_param("SecretId", if_none="")
        secret_string = self._get_param("SecretString")
        secret_binary = self._get_param("SecretBinary")
        client_request_token = self._get_param("ClientRequestToken")
        if not secret_binary and not secret_string:
            raise InvalidRequestException(
                "You must provide either SecretString or SecretBinary."
            )
        version_stages = self._get_param("VersionStages", if_none=["AWSCURRENT"])
        if not isinstance(version_stages, list):
            version_stages = [version_stages]

        return self.backend.put_secret_value(
            secret_id=secret_id,
            secret_binary=secret_binary,
            secret_string=secret_string,
            version_stages=version_stages,
            client_request_token=client_request_token,
        )

    def list_secret_version_ids(self) -> str:
        secret_id = self._get_param("SecretId", if_none="")
        return self.backend.list_secret_version_ids(secret_id=secret_id)

    def list_secrets(self) -> str:
        filters = self._get_param("Filters", if_none=[])
        _validate_filters(filters)
        max_results = self._get_int_param("MaxResults")
        next_token = self._get_param("NextToken")
        include_planned_deletion = self._get_param(
            "IncludePlannedDeletion", if_none=False
        )
        secret_list, next_token = self.backend.list_secrets(
            filters=filters,
            max_results=max_results,
            next_token=next_token,
            include_planned_deletion=include_planned_deletion,
        )
        return json.dumps(dict(SecretList=secret_list, NextToken=next_token))

    def delete_secret(self) -> str:
        secret_id = self._get_param("SecretId")
        recovery_window_in_days = self._get_param("RecoveryWindowInDays")
        force_delete_without_recovery = self._get_param("ForceDeleteWithoutRecovery")
        arn, name, deletion_date = self.backend.delete_secret(
            secret_id=secret_id,
            recovery_window_in_days=recovery_window_in_days,
            force_delete_without_recovery=force_delete_without_recovery,
        )
        return json.dumps(dict(ARN=arn, Name=name, DeletionDate=deletion_date))

    def restore_secret(self) -> str:
        secret_id = self._get_param("SecretId")
        arn, name = self.backend.restore_secret(secret_id=secret_id)
        return json.dumps(dict(ARN=arn, Name=name))

    def get_resource_policy(self) -> str:
        secret_id = self._get_param("SecretId")
        return self.backend.get_resource_policy(secret_id=secret_id)

    def put_resource_policy(self) -> str:
        secret_id = self._get_param("SecretId")
        policy = self._get_param("ResourcePolicy")
        arn, name = self.backend.put_resource_policy(secret_id, policy)
        return json.dumps(dict(ARN=arn, Name=name))

    def delete_resource_policy(self) -> str:
        secret_id = self._get_param("SecretId")
        arn, name = self.backend.delete_resource_policy(secret_id)
        return json.dumps(dict(ARN=arn, Name=name))

    def tag_resource(self) -> str:
        secret_id = self._get_param("SecretId")
        tags = self._get_param("Tags", if_none=[])
        self.backend.tag_resource(secret_id, tags)
        return "{}"

    def untag_resource(self) -> str:
        secret_id = self._get_param("SecretId")
        tag_keys = self._get_param("TagKeys", if_none=[])
        self.backend.untag_resource(secret_id=secret_id, tag_keys=tag_keys)
        return "{}"

    def update_secret_version_stage(self) -> str:
        secret_id = self._get_param("SecretId")
        version_stage = self._get_param("VersionStage")
        remove_from_version_id = self._get_param("RemoveFromVersionId")
        move_to_version_id = self._get_param("MoveToVersionId")
        arn, name = self.backend.update_secret_version_stage(
            secret_id=secret_id,
            version_stage=version_stage,
            remove_from_version_id=remove_from_version_id,
            move_to_version_id=move_to_version_id,
        )
        return json.dumps({"ARN": arn, "Name": name})

    def replicate_secret_to_regions(self) -> str:
        secret_id = self._get_param("SecretId")
        replica_regions = self._get_param("AddReplicaRegions", [])
        force_overwrite = self._get_bool_param("ForceOverwriteReplicaSecret", False)

        arn, statuses = self.backend.replicate_secret_to_regions(
            secret_id, replica_regions, force_overwrite=force_overwrite
        )
        return json.dumps({"ARN": arn, "ReplicationStatus": statuses})

    def remove_regions_from_replication(self) -> str:
        secret_id = self._get_param("SecretId")
        replica_regions = self._get_param("RemoveReplicaRegions", [])

        arn, statuses = self.backend.remove_regions_from_replication(
            secret_id, replica_regions
        )
        return json.dumps({"ARN": arn, "ReplicationStatus": statuses})



from .responses import MarketplaceMeteringResponse

url_bases = [
    r"https?://metering\.marketplace\.(.+)\.amazonaws\.com",
    r"https?://aws-marketplace\.(.+)\.amazonaws\.com",
]

url_paths = {"{0}/$": MarketplaceMeteringResponse.dispatch}



import boto3
import pytest

from moto import mock_aws


@pytest.fixture(scope="function", name="ec2")
def fixture_ec2():
    with mock_aws():
        yield boto3.client("ec2", region_name="us-east-1")


@pytest.fixture(scope="function", name="efs")
def fixture_efs():
    with mock_aws():
        yield boto3.client("efs", region_name="us-east-1")
